@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_shortcuteditor.c 20871 2006-04-13 21:07:50Z olivier $
+/*      $Id: xfwm4_shortcuteditor.c 22213 2006-06-28 21:18:22Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -48,9 +48,7 @@ struct _shortcut_tree_foreach_struct
 void
 cb_popup_del_menu (GtkWidget * widget, gpointer data)
 {
-    Itf *itf;
-
-    itf = (Itf *) data;
+    Itf *itf = (Itf *) data;
 
     if (xfce_confirm (_("Do you really want to remove this keybinding theme ?"), GTK_STOCK_YES,
             NULL))
@@ -112,6 +110,7 @@ cb_popup_del_menu (GtkWidget * widget, gpointer data)
 void
 cb_popup_add_menu (GtkWidget * widget, gpointer data)
 {
+    Itf *itf = (Itf *) data;
     gchar buf[80];
     GtkWidget *dialog;
     GtkWidget *hbox;
@@ -122,7 +121,6 @@ cb_popup_add_menu (GtkWidget * widget, gpointer data)
     GtkTreeSelection *selection;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    Itf *itf;
     ThemeInfo *ti;
 
     gchar *new_theme_path = NULL;
@@ -133,8 +131,6 @@ cb_popup_add_menu (GtkWidget * widget, gpointer data)
     FILE *new_theme;
     FILE *default_theme;
 
-    itf = (Itf *) data;
-
     dialog = gtk_dialog_new_with_buttons (_("Add keybinding theme"), GTK_WINDOW (itf->xfwm4_dialog),
         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
@@ -256,9 +252,7 @@ cb_popup_add_menu (GtkWidget * widget, gpointer data)
 gboolean
 cb_popup_menu (GtkTreeView * treeview, GdkEventButton * event, gpointer data)
 {
-    Itf *itf;
-
-    itf = (Itf *) data;
+    Itf *itf = (Itf *) data;
 
     /* Right click draws the context menu */
     if ((event->button == 3) && (event->type == GDK_BUTTON_PRESS))
@@ -325,6 +319,7 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
         "hide_window_key",
         "shade_window_key",
         "stick_window_key",
+        "cycle_windows_key",
         "move_window_up_key",
         "move_window_down_key",
         "move_window_left_key",
@@ -333,6 +328,7 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
         "resize_window_down_key",
         "resize_window_left_key",
         "resize_window_right_key",
+        "move_resize_cancel_key",
         "raise_window_key",
         "lower_window_key",
         "fullscreen_key",
@@ -361,6 +357,7 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
         N_("Hide window"),
         N_("Shade window"),
         N_("Stick window"),
+        N_("Cycle windows"),
         N_("Move window up"),
         N_("Move window down"),
         N_("Move window left"),
@@ -369,6 +366,7 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
         N_("Resize window down"),
         N_("Resize window left"),
         N_("Resize window right"),
+        N_("Cancel move/resize window"),
         N_("Raise window"),
         N_("Lower window"),
         N_("Toggle fullscreen"),
@@ -506,13 +504,6 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
             g_free (option);
         }
 
-        if (!found)
-        {
-            gtk_list_store_append (GTK_LIST_STORE (model3), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (model3), &iter, COLUMN_COMMAND, *shortcut,
-                COLUMN_SHORTCUT, entry_value, -1);
-        }
-
         shortcut++;
     }
 
@@ -663,11 +654,13 @@ is_modifier (guint keycode)
 
     keymap = XGetModifierMapping (gdk_display);
     for (n = 0; n < keymap->max_keypermod * 8; ++n)
+    {
         if (keycode == keymap->modifiermap[n])
         {
             result = TRUE;
             break;
         }
+    }
 
     XFreeModifiermap (keymap);
 
@@ -679,14 +672,15 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
 {
     Itf *itf = (Itf *) data;
 
+    GdkModifierType consumed_modifiers = 0;
     gchar shortcut_string[80] = "";
+    ThemeInfo *ti;
     GtkTreeModel *model;
     GtkTreeSelection *selection;
     GtkTreeIter iter;
     shortcut_tree_foreach_struct stfs;
-    ThemeInfo *ti;
-    GdkModifierType consumed_modifiers = 0;
     guint keyval;
+    guint modifiers;
     gchar *accelerator;
     gint i;
     gchar **shortcuts;
@@ -700,13 +694,45 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
 
     keyval = gdk_keyval_to_lower (event->keyval);
 
-    if (keyval == GDK_ISO_Left_Tab)
-        keyval = GDK_Tab;
+    switch (keyval)
+    {
+        case GDK_ISO_Left_Tab:
+            keyval = GDK_Tab;
+            break;
+        case GDK_ISO_Level3_Latch:
+        case GDK_ISO_Level3_Lock:
+        case GDK_ISO_Level3_Shift:
+        case GDK_Scroll_Lock:
+            return TRUE;
+            break;
+        default:
+            break;
+    }
 
     /* Release keyboard */
     gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 
-    accelerator = gtk_accelerator_name (keyval, event->state);
+    /*
+     * gtk_accelerator_get_default_mod_mask () limits the number of modifiers.
+     * That's a good thing because we don't want the "fake" modifiers such as
+     * Scroll/Num/Caps Lock to be taken into account.
+     *
+     * Unfortunately, at this time of writing, gdk doesn't know about all of the
+     * modifiers, like META, SUPER or HYPER, but just SHIFT, ALT and CONTROL...
+     *
+     * It means that ppl won't be able to use the "Windows" key as a modifier...
+     * Too bad, that's life, you may gently ask the gtk+ maintainers about it.
+     *
+     * Things may change in the future? See this thread:
+     *
+     * http://mail.gnome.org/archives/gtk-devel-list/2005-September/msg00024.html
+     *
+     * (Olivier)
+     */
+
+    modifiers = event->state & (~consumed_modifiers | GDK_MODIFIER_MASK);
+    modifiers = modifiers & gtk_accelerator_get_default_mod_mask ();
+    accelerator = gtk_accelerator_name (keyval, modifiers);
 
     for (i = 0; i < strlen (accelerator); i++)
     {
@@ -719,7 +745,7 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
     current_shortcut = shortcuts;
     while (*current_shortcut)
     {
-        if (strlen (*current_shortcut) > 0 && (strcmp (*current_shortcut, "Mod2") != 0))
+        if (strlen (*current_shortcut))
         {
             strcat (shortcut_string, *current_shortcut);
             strcat (shortcut_string, "+");
@@ -855,7 +881,7 @@ cb_activate_treeview3 (GtkWidget * treeview, GtkTreePath * path, GtkTreeViewColu
     g_signal_connect (G_OBJECT (dialog), "key-press-event", G_CALLBACK (cb_compose_shortcut), itf);
 
     /* Take control on the keyboard */
-    if (gdk_keyboard_grab (gtk_widget_get_root_window (label), TRUE,
+    if (gdk_keyboard_grab (gtk_widget_get_root_window (dialog), TRUE,
             GDK_CURRENT_TIME) != GDK_GRAB_SUCCESS)
     {
         g_warning ("Cannot grab the keyboard");
