@@ -1,4 +1,4 @@
-/*      $Id: compositor.c 22292 2006-07-08 12:32:54Z olivier $
+/*      $Id: compositor.c 22955 2006-08-30 19:53:33Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -66,6 +66,7 @@
 /* Some convenient macros */
 #define WIN_HAS_FRAME(cw)               ((cw->c) && FLAG_TEST (cw->c->xfwm_flags, XFWM_FLAG_HAS_BORDER) && \
                                          !FLAG_TEST (cw->c->flags, CLIENT_FLAG_FULLSCREEN))
+#define WIN_NO_SHADOW(cw)               ((cw->c) &&  FLAG_TEST (cw->c->flags, CLIENT_FLAG_FULLSCREEN | CLIENT_FLAG_BELOW))
 #define WIN_IS_OVERRIDE(cw)             (cw->c == NULL)
 #define WIN_IS_ARGB(cw)                 (cw->argb)
 #define WIN_IS_OPAQUE(cw)               (((cw->opacity == NET_WM_OPAQUE) && !WIN_IS_ARGB(cw)) || (cw->screen_info->overlays))
@@ -642,7 +643,7 @@ client_size (CWindow *cw)
     {
         XRectangle  r;
         Client *c;
-        
+
         c = cw->c;
         r.x = frameX (c) + frameLeft (c);
         r.y = frameY (c) + frameTop (c);
@@ -676,6 +677,89 @@ border_size (CWindow *cw)
     return border;
 }
 
+static void
+free_win_data (CWindow *cw, gboolean delete)
+{
+    DisplayInfo *display_info;
+    ScreenInfo *screen_info;
+
+    screen_info = cw->screen_info;
+    display_info = screen_info->display_info;
+
+#if HAVE_NAME_WINDOW_PIXMAP
+    if (cw->name_window_pixmap)
+    {
+        XFreePixmap (display_info->dpy, cw->name_window_pixmap);
+        cw->name_window_pixmap = None;
+    }
+#endif
+
+    if (cw->picture)
+    {
+        XRenderFreePicture (display_info->dpy, cw->picture);
+        cw->picture = None;
+    }
+
+    if (cw->shadow)
+    {
+        XRenderFreePicture (display_info->dpy, cw->shadow);
+        cw->shadow = None;
+    }
+
+    if (cw->alphaPict)
+    {
+        XRenderFreePicture (display_info->dpy, cw->alphaPict);
+        cw->alphaPict = None;
+    }
+
+    if (cw->shadowPict)
+    {
+        XRenderFreePicture (display_info->dpy, cw->shadowPict);
+        cw->shadowPict = None;
+    }
+
+    if (cw->alphaBorderPict)
+    {
+        XRenderFreePicture (display_info->dpy, cw->alphaBorderPict);
+        cw->alphaBorderPict = None;
+    }
+
+    if (cw->clientSize)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->clientSize);
+        cw->clientSize = None;
+    }
+
+    if (cw->borderSize)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->borderSize);
+        cw->borderSize = None;
+    }
+
+    if (cw->extents)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->extents);
+        cw->extents = None;
+    }
+
+    if (cw->borderClip)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->borderClip);
+        cw->borderClip = None;
+    }
+
+    if (delete)
+    {
+        if (cw->damage != None)
+        {
+            XDamageDestroy (display_info->dpy, cw->damage);
+            cw->damage = None;
+        }
+
+        g_free (cw);
+    }
+}
+
 static Picture
 root_tile (ScreenInfo *screen_info)
 {
@@ -686,7 +770,7 @@ root_tile (ScreenInfo *screen_info)
     gboolean fill = FALSE;
     XRenderPictureAttributes pa;
     XRenderPictFormat *format;
-#ifdef MONITOR_ROOT_PIXMAP    
+#ifdef MONITOR_ROOT_PIXMAP
     gint p;
     Atom backgroundProps[2];
 #endif
@@ -697,7 +781,7 @@ root_tile (ScreenInfo *screen_info)
     display_info = screen_info->display_info;
     dpy = display_info->dpy;
     pixmap = None;
-#ifdef MONITOR_ROOT_PIXMAP    
+#ifdef MONITOR_ROOT_PIXMAP
     backgroundProps[0] = display_info->atoms[XROOTPMAP];
     backgroundProps[1] = display_info->atoms[XSETROOT];
 
@@ -749,6 +833,7 @@ root_tile (ScreenInfo *screen_info)
         c.alpha = 0xffff;
         XRenderFillRectangle (dpy, PictOpSrc, picture, &c, 0, 0, 1, 1);
     }
+    XFreePixmap (dpy, pixmap);
     return picture;
 }
 
@@ -840,12 +925,13 @@ win_extents (CWindow *cw)
      */
 
     if (!(screen_info->overlays) &&
-         ((WIN_IS_OVERRIDE(cw) && 
-             !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw)) && 
-             screen_info->params->show_popup_shadow) || 
-         (!WIN_IS_OVERRIDE(cw) && 
-             (WIN_HAS_FRAME(cw) || !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw))) && 
-             screen_info->params->show_frame_shadow)))
+         ((screen_info->params->show_popup_shadow && 
+              WIN_IS_OVERRIDE(cw) && 
+              !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw))) || 
+          (screen_info->params->show_frame_shadow &&
+              !WIN_IS_OVERRIDE(cw) && 
+              !WIN_NO_SHADOW(cw) &&
+              (WIN_HAS_FRAME(cw) || !(WIN_IS_ARGB(cw) || WIN_IS_SHAPED(cw))))))
     {
         XRectangle sr;
 
@@ -989,13 +1075,7 @@ unredirect_win (CWindow *cw)
         screen_info = cw->screen_info;
         display_info = screen_info->display_info;
 
-#if HAVE_NAME_WINDOW_PIXMAP
-        if (cw->name_window_pixmap)
-        {
-            XFreePixmap (display_info->dpy, cw->name_window_pixmap);
-            cw->name_window_pixmap = None;
-        }
-#endif
+        free_win_data (cw, FALSE);
         cw->ignore_unmaps++;
         cw->redirected = FALSE;
 
@@ -1012,7 +1092,6 @@ paint_win (CWindow *cw, XserverRegion region, gboolean solid_part)
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
     gboolean paint_solid;
-    gint x, y, w, h;
 
     g_return_if_fail (cw != NULL);
     TRACE ("entering paint_win: 0x%lx", cw->id);
@@ -1020,32 +1099,23 @@ paint_win (CWindow *cw, XserverRegion region, gboolean solid_part)
     screen_info = cw->screen_info;
     display_info = screen_info->display_info;
     paint_solid = ((solid_part) && WIN_IS_OPAQUE(cw));
-    get_paint_bounds (cw, &x, &y, &w, &h);
 
-    if ((cw->clientSize) && (screen_info->params->frame_opacity < 100))
+    if (WIN_HAS_FRAME(cw) && (screen_info->params->frame_opacity < 100))
     {
-        XserverRegion frameClip;
+        int frame_x, frame_y, frame_width, frame_height;
+        int frame_top, frame_bottom, frame_left, frame_right;
 
-        frameClip = XFixesCreateRegion (display_info->dpy, NULL, 0);
+        frame_x = frameX (cw->c);
+        frame_y = frameY (cw->c);
+        frame_width = frameWidth (cw->c);
+        frame_height = frameHeight (cw->c);
+        frame_top = frameTop (cw->c);
+        frame_bottom = frameBottom (cw->c);
+        frame_left = frameLeft (cw->c);
+        frame_right = frameRight (cw->c);
 
-        /* Client Window */
-        if (solid_part)
+        if (!solid_part)
         {
-            /* Client */
-            if (WIN_IS_OPAQUE(cw))
-            {
-                XFixesIntersectRegion (display_info->dpy, frameClip, region, cw->clientSize);
-                XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, frameClip);
-
-                XRenderComposite (display_info->dpy, PictOpSrc, cw->picture, None,
-                                 screen_info->rootBuffer, 0, 0, 0, 0, x, y, w, h);
-
-                XFixesSubtractRegion (display_info->dpy, region, region, cw->clientSize);
-            }
-        }
-        else
-        {
-            /* Frame */
             if (!cw->alphaBorderPict)
             {
                 double frame_opacity;
@@ -1055,44 +1125,87 @@ paint_win (CWindow *cw, XserverRegion region, gboolean solid_part)
 
                 cw->alphaBorderPict = solid_picture (screen_info, FALSE, frame_opacity, 0, 0, 0);
             }
-            XFixesSubtractRegion (display_info->dpy, frameClip, cw->borderClip, cw->clientSize);
-            XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, frameClip);
 
+            /* Top Border (title bar) */
             XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
-                              screen_info->rootBuffer, 0, 0, 0, 0, x, y, w, h);
+                              screen_info->rootBuffer, 
+                              0, 0, 
+                              0, 0, 
+                              frame_x, frame_y, 
+                              frame_width, frame_top);
 
-            /* Client */
-            if (!WIN_IS_OPAQUE(cw))
-            {
-                XFixesIntersectRegion (display_info->dpy, frameClip, cw->borderClip, cw->clientSize);
-                XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, frameClip);
+            /* Bottom Border */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              0, frame_height - frame_bottom, 
+                              0, 0,
+                              frame_x, frame_y + frame_height - frame_bottom, 
+                              frame_width, frame_bottom);
+            /* Left Border */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              0, frame_top, 
+                              0, 0,
+                              frame_x, frame_y + frame_top, 
+                              frame_left, frame_height - frame_top - frame_bottom);
 
-                XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaPict,
-                                 screen_info->rootBuffer, 0, 0, 0, 0, x, y, w, h);
-            }
+            /* Right Border */
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaBorderPict,
+                              screen_info->rootBuffer, 
+                              frame_width - frame_right, frame_top, 
+                              0, 0,
+                              frame_x + frame_width - frame_right, 
+                              frame_y + frame_top, frame_right, 
+                              frame_height - frame_top - frame_bottom);
         }
+        /* Client Window */
+        if (paint_solid)
+        {
+            XRectangle  r;
+            XserverRegion client_region;
 
-        XFixesDestroyRegion (display_info->dpy, frameClip);
+            XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, region);
+            XRenderComposite (display_info->dpy, PictOpSrc, cw->picture, None,
+                              screen_info->rootBuffer, 
+                              frame_left, frame_top, 
+                              0, 0,
+                              frame_x + frame_left, frame_y + frame_top, 
+                              frame_width - frame_left - frame_right, frame_height - frame_top - frame_bottom);
+
+            r.x = frame_x + frame_left;
+            r.y = frame_y + frame_top;
+            r.width = frame_width - frame_left - frame_right;
+            r.height = frame_height - frame_top - frame_bottom;
+            client_region = XFixesCreateRegion (display_info->dpy, &r, 1);
+            XFixesSubtractRegion (display_info->dpy, region, region, client_region);
+            XFixesDestroyRegion (display_info->dpy, client_region);
+        }
+        else if (!solid_part)
+        {
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaPict,
+                              screen_info->rootBuffer, 
+                              frame_left, frame_top, 
+                              0, 0,
+                              frame_x + frame_left, frame_y + frame_top, 
+                              frame_width - frame_left - frame_right, frame_height - frame_top - frame_bottom);
+        }
     }
     else
     {
-        if (solid_part)
+        gint x, y, w, h;
+
+        get_paint_bounds (cw, &x, &y, &w, &h);
+        if (paint_solid)
         {
-            if (WIN_IS_OPAQUE(cw))
-            {
-                XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, region);
-                XRenderComposite (display_info->dpy, PictOpSrc, cw->picture, None, screen_info->rootBuffer,
-                                  0, 0, 0, 0, x, y, w, h);
-                XFixesSubtractRegion (display_info->dpy, region, region, cw->borderSize);
-            }
+            XFixesSetPictureClipRegion (display_info->dpy, screen_info->rootBuffer, 0, 0, region);
+            XRenderComposite (display_info->dpy, PictOpSrc, cw->picture, None, screen_info->rootBuffer,
+                              0, 0, 0, 0, x, y, w, h);
+            XFixesSubtractRegion (display_info->dpy, region, region, cw->borderSize);
         }
-        else
+        else if (!solid_part)
         {
-            if (!WIN_IS_OPAQUE(cw))
-            {
-                XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaPict, 
-                                  screen_info->rootBuffer, 0, 0, 0, 0, x, y, w, h);
-            }
+            XRenderComposite (display_info->dpy, PictOpOver, cw->picture, cw->alphaPict, screen_info->rootBuffer, 
+                              0, 0, 0, 0, x, y, w, h);
         }
     }
 }
@@ -1367,89 +1480,6 @@ add_repair (DisplayInfo *display_info)
 #endif /* USE_IDLE_REPAINT */
 }
 
-static void
-free_win_data (CWindow *cw, gboolean delete)
-{
-    DisplayInfo *display_info;
-    ScreenInfo *screen_info;
-
-    screen_info = cw->screen_info;
-    display_info = screen_info->display_info;
-
-#if HAVE_NAME_WINDOW_PIXMAP
-    if (cw->name_window_pixmap)
-    {
-        XFreePixmap (display_info->dpy, cw->name_window_pixmap);
-        cw->name_window_pixmap = None;
-    }
-#endif
-
-    if (cw->picture)
-    {
-        XRenderFreePicture (display_info->dpy, cw->picture);
-        cw->picture = None;
-    }
-
-    if (cw->shadow)
-    {
-        XRenderFreePicture (display_info->dpy, cw->shadow);
-        cw->shadow = None;
-    }
-
-    if (cw->alphaPict)
-    {
-        XRenderFreePicture (display_info->dpy, cw->alphaPict);
-        cw->alphaPict = None;
-    }
-
-    if (cw->shadowPict)
-    {
-        XRenderFreePicture (display_info->dpy, cw->shadowPict);
-        cw->shadowPict = None;
-    }
-
-    if (cw->alphaBorderPict)
-    {
-        XRenderFreePicture (display_info->dpy, cw->alphaBorderPict);
-        cw->alphaBorderPict = None;
-    }
-
-    if (cw->clientSize)
-    {
-        XFixesDestroyRegion (display_info->dpy, cw->clientSize);
-        cw->clientSize = None;
-    }
-
-    if (cw->borderSize)
-    {
-        XFixesDestroyRegion (display_info->dpy, cw->borderSize);
-        cw->borderSize = None;
-    }
-
-    if (cw->extents)
-    {
-        XFixesDestroyRegion (display_info->dpy, cw->extents);
-        cw->extents = None;
-    }
-
-    if (cw->borderClip)
-    {
-        XFixesDestroyRegion (display_info->dpy, cw->borderClip);
-        cw->borderClip = None;
-    }
-
-    if (delete)
-    {
-        if (cw->damage != None)
-        {
-            XDamageDestroy (display_info->dpy, cw->damage);
-            cw->damage = None;
-        }
-
-        g_free (cw);
-    }
-}
-
 static void
 add_damage (ScreenInfo *screen_info, XserverRegion damage)
 {
@@ -1493,7 +1523,7 @@ fix_region (CWindow *cw, XserverRegion region)
     for (index = screen_info->cwindows; index; index = g_list_next (index))
     {
         CWindow *cw2;
-        
+
         cw2 = (CWindow *) index->data;
         if (cw2 == cw)
         {
@@ -1596,6 +1626,30 @@ damage_win (CWindow *cw)
     add_damage (cw->screen_info, extents);
 }
 
+static void
+update_extents (CWindow *cw)
+{
+    DisplayInfo *display_info;
+    ScreenInfo *screen_info;
+
+    g_return_if_fail (cw != NULL);
+    TRACE ("entering update_extents");
+
+    screen_info = cw->screen_info;
+    display_info = screen_info->display_info;
+
+    if (WIN_IS_VISIBLE(cw))
+    {
+        damage_win (cw);
+    }
+
+    if (cw->extents)
+    {
+        XFixesDestroyRegion (display_info->dpy, cw->extents);
+        cw->extents = None;
+    }
+}
+
 static void
 determine_mode(CWindow *cw)
 {
@@ -1699,7 +1753,7 @@ map_win (CWindow *cw)
         cw->ignore_unmaps++;
         return;
     }
-    
+
     screen_info = cw->screen_info;
     if (!WIN_IS_REDIRECTED(cw))
     {
@@ -1708,16 +1762,22 @@ map_win (CWindow *cw)
         return;
     }
 
+    if (!screen_info->params->unredirect_overlays)
+    {
+        TRACE ("Not unredirecting overlays");
+        return;
+    }
+
     /* Check for new windows to un-redirect. */
-    if (WIN_IS_FULLSCREEN(cw) &&  WIN_HAS_DAMAGE(cw) &&  WIN_IS_OVERRIDE(cw) && 
-        WIN_IS_NATIVE_OPAQUE(cw) &&  WIN_IS_REDIRECTED(cw) && !WIN_IS_SHAPED(cw))
+    if (WIN_IS_FULLSCREEN(cw) && WIN_HAS_DAMAGE(cw) && WIN_IS_OVERRIDE(cw) && 
+        WIN_IS_NATIVE_OPAQUE(cw) && WIN_IS_REDIRECTED(cw) && !WIN_IS_SHAPED(cw))
     {
         CWindow *top;
         GList *index;
 
         index = screen_info->cwindows;
         top = (CWindow *) index->data;
-        
+
         if (cw == top)
         {
             TRACE ("Toplevel window 0x%lx is fullscreen, unredirecting", cw->id);
@@ -1766,9 +1826,9 @@ init_opacity (CWindow *cw)
 
     TRACE ("set_opacity");
     g_return_if_fail (cw != NULL);
-    
+
     screen_info = cw->screen_info;
-    display_info = screen_info->display_info;    
+    display_info = screen_info->display_info;
     c = cw->c;
 
     cw->native_opacity = FALSE;
@@ -1796,7 +1856,7 @@ add_win (DisplayInfo *display_info, Window id, Client *c)
 {
     ScreenInfo *screen_info;
     CWindow *new;
-        
+
     TRACE ("entering add_win: 0x%lx", id);
 
     new = g_new0 (CWindow, 1);
@@ -1919,7 +1979,16 @@ restack_win (CWindow *cw, Window above)
         previous_above = ncw->id;
     }
 
-    if (previous_above != above)
+    /* If above is set to None, the window whose state was changed is on 
+     * the bottom of the stack with respect to sibling.
+     */
+    if (above == None)
+    {
+        /* Insert at bottom of window stack */
+        screen_info->cwindows = g_list_delete_link (screen_info->cwindows, sibling);
+        screen_info->cwindows = g_list_append (screen_info->cwindows, cw);
+    }
+    else if (previous_above != above)
     {
         GList *index;
 
@@ -1937,19 +2006,6 @@ restack_win (CWindow *cw, Window above)
             screen_info->cwindows = g_list_delete_link (screen_info->cwindows, sibling);
             screen_info->cwindows = g_list_insert_before (screen_info->cwindows, index, cw);
         }
-        else if (above == None)
-        {
-            /* Insert at bottom of window stack */
-            screen_info->cwindows = g_list_delete_link (screen_info->cwindows, sibling);
-            screen_info->cwindows = g_list_append (screen_info->cwindows, cw);
-        }
-        else
-        {
-            /* Don't know what to do */
-            g_warning ("The window 0x%lx has not been restacked\n"
-                       "because the specified sibling 0x%lx was "
-                       "not found in our stack", cw->id, above);
-        }
     }
 }
 
@@ -1974,12 +2030,6 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
     TRACE ("entering resize_win");
     TRACE ("resizing 0x%lx, (%i,%i) %ix%i", cw->id, x, y, width, height);
 
-    if (!WIN_IS_VISIBLE(cw))
-    {
-        set_size_attributes (cw, x, y, width, height, bw);
-        return;
-    }
-
     screen_info = cw->screen_info;
     display_info = screen_info->display_info;
 
@@ -2000,6 +2050,10 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
             cw->name_window_pixmap = None;
         }
 #endif
+    }
+
+    if ((cw->attr.width != width) || (cw->attr.height != height) || shape_notify)
+    {
         if (cw->picture)
         {
             XRenderFreePicture (display_info->dpy, cw->picture);
@@ -2013,7 +2067,8 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
         }
     }
 
-    if ((cw->attr.width != width) || (cw->attr.height != height) || (cw->attr.x != x) || (cw->attr.y != y))
+    if ((cw->attr.width != width) || (cw->attr.height != height) || 
+        (cw->attr.x != x) || (cw->attr.y != y) || shape_notify)
     {
         if (cw->borderSize)
         {
@@ -2030,6 +2085,12 @@ resize_win (CWindow *cw, gint x, gint y, gint width, gint height, gint bw, gbool
 
     set_size_attributes (cw, x, y, width, height, bw);
 
+    if (!WIN_IS_VISIBLE(cw))
+    {
+        /* 'nuff for invisible windows... */
+        return;
+    }
+
     cw->extents = win_extents (cw);
     XFixesUnionRegion (display_info->dpy, damage, damage, cw->extents);
     /* A shape notify will likely change the shadows too, so clear the extents */
@@ -2047,7 +2108,7 @@ static void
 destroy_win (DisplayInfo *display_info, Window id)
 {
     CWindow *cw;
-
+ 
     g_return_if_fail (display_info != NULL);
     g_return_if_fail (id != None);
     TRACE ("entering destroy_win: 0x%lx", id);
@@ -2085,7 +2146,7 @@ compositorHandleDamage (DisplayInfo *display_info, XDamageNotifyEvent *ev)
 static void
 compositorHandlePropertyNotify (DisplayInfo *display_info, XPropertyEvent *ev)
 {
-#ifdef MONITOR_ROOT_PIXMAP    
+#ifdef MONITOR_ROOT_PIXMAP
     gint p;
     Atom backgroundProps[2];
 #endif
@@ -2094,7 +2155,7 @@ compositorHandlePropertyNotify (DisplayInfo *display_info, XPropertyEvent *ev)
     g_return_if_fail (ev != NULL);
     TRACE ("entering compositorHandlePropertyNotify for 0x%lx", ev->window);
 
-#ifdef MONITOR_ROOT_PIXMAP    
+#ifdef MONITOR_ROOT_PIXMAP
     backgroundProps[0] = display_info->atoms[XROOTPMAP];
     backgroundProps[1] = display_info->atoms[XSETROOT];
 
@@ -2125,7 +2186,7 @@ compositorHandlePropertyNotify (DisplayInfo *display_info, XPropertyEvent *ev)
         if (cw)
         {
             Client *c = cw->c;
-            
+
             TRACE ("Opacity changed for 0x%lx", cw->id);
             if (!getOpacity (display_info, cw->id, &cw->opacity))
             {
@@ -2263,7 +2324,7 @@ compositorHandleCreateNotify (DisplayInfo *display_info, XCreateWindowEvent *ev)
        We are only interested in top level windows, other will
        be caught by the WM.
      */
-    
+
     if (myDisplayGetScreenFromRoot (display_info, ev->parent) != NULL)
     {
         if (!find_cwindow_in_display (display_info, ev->window))
@@ -2405,24 +2466,42 @@ compositorAddWindow (DisplayInfo *display_info, Window id, Client *c)
     }
 
     cw = find_cwindow_in_display (display_info, id);
-    if (cw)
-    {
-        /* 
-         * The compositor window is already known, just update the client if
-         * that is meaningfull...
-         */
-        if (c)
-        {
-            cw->c = c;
-        }
-    }
-    else
+    if (!compositorSetClient (display_info, id, c))
     {
         add_win (display_info, id, c);
     }
 #endif /* HAVE_COMPOSITOR */
 }
 
+gboolean
+compositorSetClient (DisplayInfo *display_info, Window id, Client *c)
+{
+#ifdef HAVE_COMPOSITOR
+    CWindow *cw;
+
+    g_return_val_if_fail (display_info != NULL, FALSE);
+    g_return_val_if_fail (id != None, FALSE);
+    TRACE ("entering compositorSetClient: 0x%lx", id);
+
+    if (!compositorIsUsable (display_info))
+    {
+        return FALSE;
+    }
+
+    cw = find_cwindow_in_display (display_info, id);
+    if (cw)
+    {
+        if (cw->c != c)
+        {
+            update_extents (cw);
+            cw->c = c;
+        }
+        return TRUE;
+    }
+#endif /* HAVE_COMPOSITOR */
+    return FALSE;
+}
+
 void
 compositorRemoveWindow (DisplayInfo *display_info, Window id)
 {
@@ -2440,6 +2519,30 @@ compositorRemoveWindow (DisplayInfo *display_info, Window id)
 #endif /* HAVE_COMPOSITOR */
 }
 
+void
+compositorDamageWindow (DisplayInfo *display_info, Window id)
+{
+#ifdef HAVE_COMPOSITOR
+    CWindow *cw;
+
+    g_return_if_fail (display_info != NULL);
+    g_return_if_fail (id != None);
+    TRACE ("entering compositorDamageWindow: 0x%lx", id);
+
+    if (!compositorIsUsable (display_info))
+    {
+        return;
+    }
+
+    cw = find_cwindow_in_display (display_info, id);
+    if (cw)
+    {
+        /* that will also damage the window */
+        update_extents (cw);
+    }
+#endif /* HAVE_COMPOSITOR */
+}
+
 void
 compositorHandleEvent (DisplayInfo *display_info, XEvent *ev)
 {
@@ -2608,6 +2711,26 @@ compositorSetCompositeMode (DisplayInfo *display_info, gboolean use_manual_redir
 #endif /* HAVE_COMPOSITOR */
 }
 
+static void
+compositorSetCMSelection (ScreenInfo *screen_info, Window w)
+{
+    DisplayInfo *display_info;
+    gchar selection[32];
+    Atom a;
+    
+    g_return_if_fail (screen_info != NULL);
+
+    display_info = screen_info->display_info;
+    /* Newer EWMH standard property "_NET_WM_CM_S<n>" */
+    g_snprintf (selection, sizeof (selection), "_NET_WM_CM_S%d", screen_info->screen);
+    a = XInternAtom (display_info->dpy, selection, FALSE);
+    setXAtomManagerOwner (display_info, a, screen_info->xroot, w);
+
+    /* Older property "COMPOSITING_MANAGER" */
+    setAtomIdManagerOwner (display_info, COMPOSITING_MANAGER, screen_info->xroot, w);
+}
+
+
 gboolean
 compositorManageScreen (ScreenInfo *screen_info)
 {
@@ -2674,8 +2797,7 @@ compositorManageScreen (ScreenInfo *screen_info)
     screen_info->overlays = 0;
 
     XClearArea (display_info->dpy, screen_info->xroot, 0, 0, 0, 0, TRUE);
-    setAtomManagerOwner (display_info, COMPOSITING_MANAGER, 
-                         screen_info->xroot, screen_info->xfwm4_win);
+    compositorSetCMSelection (screen_info, screen_info->xfwm4_win);
 
     return TRUE;
 #else
@@ -2753,7 +2875,7 @@ compositorUnmanageScreen (ScreenInfo *screen_info)
     XCompositeUnredirectSubwindows (display_info->dpy, screen_info->xroot, 
                                     display_info->composite_mode);
 
-    setAtomManagerOwner (display_info, COMPOSITING_MANAGER, screen_info->xroot, None);
+    compositorSetCMSelection (screen_info, None);
 #endif /* HAVE_COMPOSITOR */
 }
 
@@ -2780,7 +2902,7 @@ compositorAddAllWindows (ScreenInfo *screen_info)
     for (i = 0; i < count; i++)
     {
         Client *c;
-        
+
         c = clientGetFromWindow (screen_info, wins[i], FRAME);
         compositorAddWindow (display_info, wins[i], c);
     }
@@ -2831,7 +2953,7 @@ compositorUpdateScreenSize (ScreenInfo *screen_info)
 {
 #ifdef HAVE_COMPOSITOR
     DisplayInfo *display_info;
-    
+
     g_return_if_fail (screen_info != NULL);
     TRACE ("entering compositorUpdateScreenSize");
 
