@@ -1,4 +1,4 @@
-/*      $Id: compositor.h 22243 2006-07-02 16:23:01Z olivier $
+/*      $Id: compositor.h 22871 2006-08-24 17:08:39Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -36,9 +36,13 @@ gboolean compositorIsUsable           (DisplayInfo *);
 void     compositorAddWindow          (DisplayInfo *,
                                        Window,
                                        Client *);
+gboolean compositorSetClient          (DisplayInfo *,
+                                       Window,
+                                       Client *);
 void     compositorRemoveWindow       (DisplayInfo *,
                                        Window);
-
+void     compositorDamageWindow       (DisplayInfo *, 
+                                       Window);
 void     compositorHandleEvent        (DisplayInfo *,
                                        XEvent *);
 void     compositorInitDisplay        (DisplayInfo *);
