@@ -1,4 +1,4 @@
-/*      $Id: display.c 20909 2006-04-15 09:08:14Z olivier $
+/*      $Id: display.c 22296 2006-07-08 16:57:14Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -29,10 +29,11 @@
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
-#include <gtk/gtk.h>
 #include <glib.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
 #ifdef HAVE_RENDER
 #include <X11/extensions/Xrender.h>
 #endif
@@ -43,6 +44,10 @@
 #include "client.h"
 #include "compositor.h"
 
+#ifndef MAX_HOSTNAME_LENGTH
+#define MAX_HOSTNAME_LENGTH 32
+#endif /* MAX_HOSTNAME_LENGTH */
+
 static int
 handleXError (Display * dpy, XErrorEvent * err)
 {
@@ -66,6 +71,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
         "KWM_WIN_ICON",
         "_MOTIF_WM_HINTS",
+        "_MOTIF_WM_INFO",
         "_NET_ACTIVE_WINDOW",
         "_NET_CLIENT_LIST",
         "_NET_CLIENT_LIST_STACKING",
@@ -100,6 +106,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_MOVERESIZE",
         "_NET_WM_NAME",
         "_NET_WM_WINDOW_OPACITY",
+        "_NET_WM_WINDOW_OPACITY_LOCKED",
         "_NET_WM_STATE",
         "_NET_WM_STATE_ABOVE",
         "_NET_WM_STATE_BELOW",
@@ -141,6 +148,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_WIN_WORKSPACE_COUNT",
         "WM_CHANGE_STATE",
         "WM_CLIENT_LEADER",
+        "WM_CLIENT_MACHINE",
         "WM_COLORMAP_WINDOWS",
         "WM_DELETE_WINDOW",
         "WM_HINTS",
@@ -149,6 +157,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "WM_TAKE_FOCUS",
         "WM_TRANSIENT_FOR",
         "WM_WINDOW_ROLE",
+        "XFWM4_COMPOSITING_MANAGER",
         "_XROOTPMAP_ID",
         "_XSETROOT_ID"
     };
@@ -171,6 +180,10 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->gdisplay = gdisplay;
     display->dpy = (Display *) gdk_x11_display_get_xdisplay (gdisplay);
 
+    display->session = NULL;
+    display->quit = FALSE;
+    display->reload = FALSE;
+
     XSetErrorHandler (handleXError);
 
     /* Initialize internal atoms */
@@ -259,6 +272,13 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->dbl_click_time = 300;
     display->nb_screens = 0;
     display->current_time = CurrentTime;
+
+    display->hostname = g_new0 (gchar, (size_t) MAX_HOSTNAME_LENGTH);
+    if (gethostname ((char *) display->hostname, MAX_HOSTNAME_LENGTH - 1))
+    {
+        g_free (display->hostname);
+        display->hostname = NULL;
+    }
     
     compositorInitDisplay (display);
 
@@ -277,6 +297,12 @@ myDisplayClose (DisplayInfo *display)
     XFreeCursor (display->dpy, display->root_cursor);
     display->root_cursor = None;
     
+    if (display->hostname)
+    {
+        g_free (display->hostname);
+        display->hostname = NULL;
+    }
+
     for (i = 0; i < 7; i++)
     {
         XFreeCursor (display->dpy, display->resize_cursor[i]);
@@ -566,21 +592,20 @@ ScreenInfo *
 myDisplayGetDefaultScreen (DisplayInfo *display)
 {
     GSList *index;
-    ScreenInfo *screen = NULL;
 
     g_return_val_if_fail (display != NULL, NULL);
 
     index = display->screens;
     if (index)
     {
-        screen = (ScreenInfo *) index->data;
+        return (ScreenInfo *) index->data;
     }
 
-    return screen;
+    return NULL;
 }
 
 Time 
-myDisplayUpdateCurentTime (DisplayInfo *display, XEvent *ev)
+myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
 {
     g_return_val_if_fail (display != NULL, (Time) CurrentTime);
     
@@ -627,6 +652,28 @@ myDisplayGetCurrentTime (DisplayInfo *display)
     return (Time) display->current_time;
 }
 
+Time          myDisplayGetLastUserTime      (DisplayInfo *);
+void          myDisplaySetLastUserTime      (DisplayInfo *,
+                                             Time);
+Time 
+myDisplayGetLastUserTime (DisplayInfo *display)
+{
+    g_return_val_if_fail (display != NULL, (Time) CurrentTime);
+
+    return (Time) display->last_user_time;
+}
+
+void 
+myDisplaySetLastUserTime (DisplayInfo *display, Time timestamp)
+{
+    g_return_if_fail (display != NULL);
+
+    if (TIMESTAMP_IS_BEFORE(display->last_user_time, timestamp))
+    {
+        display->last_user_time = timestamp;
+    }
+}
+
 static gdouble
 get_time(void)
 {
