@@ -1,4 +1,4 @@
-/*      $Id: display.h 22296 2006-07-08 16:57:14Z olivier $
+/*      $Id: display.h 22837 2006-08-19 22:16:16Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #endif
 
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
 
@@ -175,7 +176,6 @@ enum
 };
 
 typedef struct _Client            Client;
-typedef struct _ClientPixmapCache ClientPixmapCache;
 typedef struct _DisplayInfo       DisplayInfo;
 typedef struct _XfwmColor         XfwmColor;
 typedef struct _xfwmPixmap        xfwmPixmap;
