@@ -1,4 +1,4 @@
-/*      $Id: events.c 22296 2006-07-08 16:57:14Z olivier $
+/*      $Id: events.c 22847 2006-08-21 20:13:05Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -353,20 +353,17 @@ getKeyPressed (ScreenInfo *screen_info, XKeyEvent * ev)
 static void
 handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
 {
-    unsigned long mode;
     ScreenInfo *screen_info;
     Client *c;
     int key;
 
     TRACE ("entering handleKeyEvent");
 
-    mode = 0;
     c = clientGetFocus ();
     if (c)
     {
         screen_info = c->screen_info;
         key = getKeyPressed (screen_info, ev);
-        mode = c->win_state & WIN_STATE_MAXIMIZED;
 
         switch (key)
         {
@@ -398,13 +395,13 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 }
                 break;
             case KEY_MAXIMIZE_WINDOW:
-                clientToggleMaximized (c, mode ? mode : WIN_STATE_MAXIMIZED, TRUE);
+                clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
                 break;
             case KEY_MAXIMIZE_VERT:
-                clientToggleMaximized (c, mode ? mode : WIN_STATE_MAXIMIZED_VERT, TRUE);
+                clientToggleMaximized (c, WIN_STATE_MAXIMIZED_VERT, TRUE);
                 break;
             case KEY_MAXIMIZE_HORIZ:
-                clientToggleMaximized (c, mode ? mode : WIN_STATE_MAXIMIZED_HORIZ, TRUE);
+                clientToggleMaximized (c, WIN_STATE_MAXIMIZED_HORIZ, TRUE);
                 break;
             case KEY_SHADE_WINDOW:
                 clientToggleShaded (c);
@@ -413,7 +410,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 if (CLIENT_CAN_STICK_WINDOW(c))
                 {
                     clientToggleSticky (c, TRUE);
-                    frameDraw (c, FALSE, FALSE);
+                    frameDraw (c, FALSE);
                 }
                 break;
             case KEY_RAISE_WINDOW:
@@ -545,6 +542,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
         default:
             break;
     }
+    XAllowEvents (display_info->dpy, AsyncKeyboard, CurrentTime);
 }
 
 /* User has clicked on an edge or corner.
@@ -590,7 +588,6 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
 static void
 button1Action (Client * c, XButtonEvent * ev)
 {
-    unsigned long mode;
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
     XEvent copy_event;
@@ -601,7 +598,6 @@ button1Action (Client * c, XButtonEvent * ev)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
-    mode = c->win_state & WIN_STATE_MAXIMIZED;
 
     if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
     {
@@ -622,7 +618,7 @@ button1Action (Client * c, XButtonEvent * ev)
         switch (screen_info->params->double_click_action)
         {
             case ACTION_MAXIMIZE:
-                clientToggleMaximized (c, mode ? mode : WIN_STATE_MAXIMIZED, TRUE);
+                clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
                 break;
             case ACTION_SHADE:
                 clientToggleShaded (c);
@@ -990,11 +986,11 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
 
         if (replay)
         {
-            XAllowEvents (display_info->dpy, ReplayPointer, ev->time);
+            XAllowEvents (display_info->dpy, ReplayPointer, CurrentTime);
         }
         else
         {
-            XAllowEvents (display_info->dpy, SyncPointer, ev->time);
+            XAllowEvents (display_info->dpy, SyncPointer, CurrentTime);
         }
 
         return;
@@ -1180,7 +1176,6 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
         }
 
         screen_info = c->screen_info;
-        clientPassFocus (screen_info, c, c);
 
         /*
          * ICCCM spec states that a client wishing to switch
@@ -1193,9 +1188,11 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
         if ((ev->event == screen_info->xroot) && (ev->send_event))
         {
             TRACE ("ICCCM UnmapNotify for \"%s\"", c->name);
+            clientPassFocus (screen_info, c, c);
             clientUnframe (c, FALSE);
             return;
         }
+
         if (c->ignore_unmap)
         {
             c->ignore_unmap--;
@@ -1206,6 +1203,7 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
         {
             TRACE ("unmapping \"%s\" as ignore_unmap is %i", 
                  c->name, c->ignore_unmap);
+            clientPassFocus (screen_info, c, c);
             clientUnframe (c, FALSE);
         }
     }
@@ -1271,28 +1269,10 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
 {
     Client *c = NULL;
     XWindowChanges wc;
-    XEvent otherEvent;
 
     TRACE ("entering handleConfigureRequest");
     TRACE ("ConfigureRequest on window (0x%lx)", ev->window);
 
-    /* Compress events - logic taken from kwin */
-    while (XCheckTypedWindowEvent (display_info->dpy, ev->window, ConfigureRequest, &otherEvent))
-    {
-        /* Update the display time */
-        myDisplayUpdateCurrentTime (display_info, &otherEvent);
-
-        if (otherEvent.xconfigurerequest.value_mask == ev->value_mask)
-        {
-            ev = &otherEvent.xconfigurerequest;
-        }
-        else
-        {
-            XPutBackEvent (display_info->dpy, &otherEvent);
-            break;
-        }
-    }
-
     wc.x = ev->x;
     wc.y = ev->y;
     wc.width = ev->width;
@@ -1369,6 +1349,13 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
 
             ev->value_mask |= (CWX | CWY | CWWidth | CWHeight);
         }
+        else if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+        {
+            wc.x = c->x;
+            wc.y = c->y;
+            wc.width = c->width;
+            wc.height = c->height;
+        }
         /* Clean up buggy requests that set all flags */
         if ((ev->value_mask & CWX) && (wc.x == c->x))
         {
@@ -1596,6 +1583,10 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
                 "NotifyNormal" :
                 (ev->mode == NotifyWhileGrabbed) ?
                 "NotifyWhileGrabbed" :
+                (ev->mode == NotifyGrab) ?
+                "NotifyGrab" :
+                (ev->mode == NotifyUngrab) ?
+                "NotifyUngrab" :
                 "(unknown)");
     TRACE ("handleFocusIn (0x%lx) detail = %s",
                 ev->window,
@@ -1621,7 +1612,7 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
     last_raised = NULL;
 
     if (screen_info && (ev->window == screen_info->xroot) && (ev->mode == NotifyNormal) && 
-        (ev->detail == NotifyDetailNone))
+        ((ev->detail == NotifyDetailNone) || (ev->detail == NotifyInferior)))
     {
         /* Handle focus transition to root (means that an unknown
            window has vanished and the focus is returned to the root
@@ -1641,7 +1632,7 @@ handleFocusIn (DisplayInfo *display_info, XFocusChangeEvent * ev)
         return;
     }
 
-    c = myDisplayGetClientFromWindow (display_info, ev->window, WINDOW);
+    c = myDisplayGetClientFromWindow (display_info, ev->window, ANY);
     TRACE ("FocusIn on window (0x%lx)", ev->window);
     if (c)
     {
@@ -1698,7 +1689,7 @@ handleFocusOut (DisplayInfo *display_info, XFocusChangeEvent * ev)
         && ((ev->detail == NotifyNonlinear) 
             || (ev->detail == NotifyNonlinearVirtual)))
     {
-        c = myDisplayGetClientFromWindow (display_info, ev->window, WINDOW);
+        c = myDisplayGetClientFromWindow (display_info, ev->window, ANY);
         TRACE ("FocusOut on window (0x%lx)", ev->window);
         if ((c) && (c == clientGetFocus ()))
         {
@@ -1739,7 +1730,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
             }
             getWindowName (display_info, c->window, &c->name);
             FLAG_SET (c->flags, CLIENT_FLAG_NAME_CHANGED);
-            frameDraw (c, TRUE, FALSE);
+            frameDraw (c, TRUE);
         }
         else if (ev->atom == display_info->atoms[MOTIF_WM_HINTS])
         {
@@ -1759,7 +1750,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
                 if ((c->wmhints->flags & IconPixmapHint) && (screen_info->params->show_app_icon))
                 {
                     clientUpdateIcon (c);
-                    frameDraw (c, TRUE, FALSE);
+                    frameDraw (c, TRUE);
                 }
                 if (HINTS_ACCEPT_INPUT (c->wmhints))
                 {
@@ -1798,7 +1789,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
         {
             TRACE ("client \"%s\" (0x%lx) has received a net_wm_window_type notify", c->name, c->window);
             clientGetNetWmType (c);
-            frameDraw (c, TRUE, FALSE);
+            frameDraw (c, TRUE);
         }
         else if ((ev->atom == display_info->atoms[NET_WM_STRUT]) || 
                  (ev->atom == display_info->atoms[NET_WM_STRUT_PARTIAL]))
@@ -1843,7 +1834,7 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
                   (ev->atom == display_info->atoms[KWM_WIN_ICON])))
         {
             clientUpdateIcon (c);
-            frameDraw (c, TRUE, FALSE);
+            frameDraw (c, TRUE);
         }
 #ifdef HAVE_STARTUP_NOTIFICATION
         else if (ev->atom == display_info->atoms[NET_STARTUP_ID])
@@ -1944,7 +1935,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
                     {
                         clientStick (c, TRUE);
-                        frameDraw (c, FALSE, FALSE);
+                        frameDraw (c, FALSE);
                     }
                 }
                 else
@@ -1952,7 +1943,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
                     {
                         clientUnstick (c, TRUE);
-                        frameDraw (c, FALSE, FALSE);
+                        frameDraw (c, FALSE);
                     }
                     if (ev->data.l[0] != c->win_workspace)
                     {
@@ -2078,24 +2069,33 @@ static void
 handleShape (DisplayInfo *display_info, XShapeEvent * ev)
 {
     Client *c;
+    gboolean update;
 
     TRACE ("entering handleShape");
 
     c = myDisplayGetClientFromWindow (display_info, ev->window, WINDOW);
     if (c)
     {
+        update = FALSE;
         if (ev->kind == ShapeBounding)
         {
             if ((ev->shaped) && !FLAG_TEST (c->flags, CLIENT_FLAG_HAS_SHAPE))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_HAS_SHAPE);
+                clientGetMWMHints (c, TRUE);
+                update = TRUE;
             }
             else if (!(ev->shaped) && FLAG_TEST (c->flags, CLIENT_FLAG_HAS_SHAPE))
             {
                 FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_SHAPE);
+                clientGetMWMHints (c, TRUE);
+                update = TRUE;
             }
         }
-        frameDraw (c, FALSE, TRUE);
+        if (!update)
+        {
+            frameDraw (c, FALSE);
+        }
     }
 }
 
@@ -2257,7 +2257,6 @@ xfwm4_event_filter (XEvent * xevent, gpointer data)
 static void
 menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer item_data)
 {
-    unsigned long mode;
     Client *c;
 
     TRACE ("entering menu_callback");
@@ -2277,7 +2276,6 @@ menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer
     if (c)
     {
         c->button_pressed[MENU_BUTTON] = FALSE;
-        mode = c->win_state & WIN_STATE_MAXIMIZED;
 
         switch (op)
         {
@@ -2288,7 +2286,7 @@ menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer
             case MENU_OP_UNMAXIMIZE:
                 if (CLIENT_CAN_MAXIMIZE_WINDOW (c))
                 {
-                    clientToggleMaximized (c, mode ? mode : WIN_STATE_MAXIMIZED, TRUE);
+                    clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
                 }
                 break;
             case MENU_OP_MINIMIZE:
@@ -2296,11 +2294,11 @@ menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer
                 {
                     clientHide (c, c->win_workspace, TRUE);
                 }
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 break;
             case MENU_OP_MINIMIZE_ALL:
                 clientHideAll (c, c->win_workspace);
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 break;
             case MENU_OP_UNMINIMIZE:
                 clientShow (c, TRUE);
@@ -2313,26 +2311,26 @@ menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer
             case MENU_OP_STICK:
             case MENU_OP_UNSTICK:
                 clientToggleSticky (c, TRUE);
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 break;
             case MENU_OP_WORKSPACES:
                 clientSetWorkspace (c, GPOINTER_TO_INT (item_data), TRUE);
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 break;
             case MENU_OP_DELETE:
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 clientClose (c);
                 break;
             case MENU_OP_CONTEXT_HELP:
                 clientEnterContextMenuState (c);
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 break;
             case MENU_OP_ABOVE:
             case MENU_OP_NORMAL:
                 clientToggleAbove (c);
                 /* Fall thru */
             default:
-                frameDraw (c, FALSE, FALSE);
+                frameDraw (c, FALSE);
                 break;
         }
     }
@@ -2369,7 +2367,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
     if ((c) && ((ev->button == 1) || (ev->button == 3)))
     {
         c->button_pressed[MENU_BUTTON] = TRUE;
-        frameDraw (c, FALSE, FALSE);
+        frameDraw (c, FALSE);
         y = c->y;
         ops = MENU_OP_DELETE | MENU_OP_MINIMIZE_ALL | MENU_OP_WORKSPACES;
         insensitive = 0;
@@ -2509,7 +2507,8 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
                     &menu_event_window, 0, 0, 
                     screen_info->width,
                     screen_info->height, 
-                    NoEventMask);
+                    NoEventMask,
+                    FALSE);
     menu = menu_default (screen_info->gscr, c->window, ops, insensitive, menu_callback, 
                          c->win_workspace, screen_info->workspace_count, 
                          screen_info->workspace_names, screen_info->workspace_names_items,
@@ -2520,7 +2519,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         TRACE ("Cannot open menu");
         gdk_beep ();
         c->button_pressed[MENU_BUTTON] = FALSE;
-        frameDraw (c, FALSE, FALSE);
+        frameDraw (c, FALSE);
         xfwmWindowDelete (&menu_event_window);
         menu_free (menu);
     }
