@@ -1,4 +1,4 @@
-/*      $Id: focus.c 22223 2006-06-30 09:24:40Z olivier $
+/*      $Id: focus.c 22952 2006-08-30 16:44:24Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -342,6 +342,7 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                     clientRaise (c2, None);
                 }
             }
+#if 0
             else
             {
                 c2 = clientGetNext (c, 0);
@@ -350,6 +351,7 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                     new_focus = c2;
                 }
             }
+#endif
         }
     }
     else if (XQueryPointer (myScreenGetXDisplay (screen_info), screen_info->xroot, &dr, &window, &rx, &ry, &wx, &wy, &mask))
@@ -460,7 +462,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         }
         data[0] = c->window;
         clientAdjustFullscreenLayer (c, TRUE);
-        frameDraw (c, FALSE, FALSE);
+        frameDraw (c, FALSE);
     }
     else
     {
@@ -473,7 +475,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
             clientAdjustFullscreenLayer (c2, FALSE);
             /* clientRaise (c, None); */
         }
-        frameDraw (c2, FALSE, FALSE);
+        frameDraw (c2, FALSE);
     }
     data[1] = None;
     XChangeProperty (display_info->dpy, screen_info->xroot, 
@@ -525,7 +527,18 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
         if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT) || !(screen_info->params->focus_hint))
         {
             pending_focus = c;
-            XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
+            /* 
+             * When shaded, the client window is unmapped, so it can not be focused.
+             * Instead, we focus the frame that is still mapped.
+             */
+            if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
+            {
+                XSetInputFocus (myScreenGetXDisplay (screen_info), c->frame, RevertToPointerRoot, timestamp);
+            }
+            else
+            {
+                XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
+            }
             clientUpdateOpacity (screen_info, c);
         }
         if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
@@ -543,7 +556,7 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
         client_focus = NULL;
         if (c2)
         {
-            frameDraw (c2, FALSE, FALSE);
+            frameDraw (c2, FALSE);
             XChangeProperty (clientGetXDisplay (c2), c2->screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                              PropModeReplace, (unsigned char *) data, 2);
         }
@@ -596,14 +609,7 @@ clientUngrabMouseButton (Client * c)
     TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
-    if (screen_info->params->raise_with_any_button)
-    {
-        ungrabButton(clientGetXDisplay (c), AnyButton, AnyModifier, c->window);
-    }
-    else
-    {
-        ungrabButton(clientGetXDisplay (c), Button1, AnyModifier, c->window);
-    }
+    ungrabButton(clientGetXDisplay (c), AnyButton, AnyModifier, c->window);
     /* We've ungrabbed way too much, so regrab the regular buttons/modifiers */
     clientGrabButtons (c);
 }
