@@ -1,4 +1,4 @@
-/*      $Id: frame.c 19935 2006-02-18 08:21:05Z olivier $
+/*      $Id: frame.c 22847 2006-08-21 20:13:05Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 int
 frameDecorationLeft (ScreenInfo *screen_info)
 {
-    TRACE ("entering frameLeft");
+    TRACE ("entering frameDecorationLeft");
 
     g_return_val_if_fail (screen_info != NULL, 0);
     return screen_info->sides[SIDE_LEFT][ACTIVE].width;
@@ -49,7 +49,7 @@ frameDecorationLeft (ScreenInfo *screen_info)
 int
 frameDecorationRight (ScreenInfo *screen_info)
 {
-    TRACE ("entering frameLeft");
+    TRACE ("entering frameDecorationRight");
 
     g_return_val_if_fail (screen_info != NULL, 0);
     return screen_info->sides[SIDE_RIGHT][ACTIVE].width;
@@ -58,7 +58,7 @@ frameDecorationRight (ScreenInfo *screen_info)
 int
 frameDecorationTop (ScreenInfo *screen_info)
 {
-    TRACE ("entering frameLeft");
+    TRACE ("entering frameDecorationTop");
 
     g_return_val_if_fail (screen_info != NULL, 0);
     return screen_info->title[TITLE_3][ACTIVE].height;
@@ -67,7 +67,7 @@ frameDecorationTop (ScreenInfo *screen_info)
 int
 frameDecorationBottom (ScreenInfo *screen_info)
 {
-    TRACE ("entering frameLeft");
+    TRACE ("entering frameDecorationBottom");
 
     g_return_val_if_fail (screen_info != NULL, 0);
     return screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
@@ -80,7 +80,8 @@ frameLeft (Client * c)
 
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
     {
         return c->screen_info->sides[SIDE_LEFT][ACTIVE].width;
     }
@@ -94,7 +95,8 @@ frameRight (Client * c)
 
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
     {
         return c->screen_info->sides[SIDE_RIGHT][ACTIVE].width;
     }
@@ -122,7 +124,8 @@ frameBottom (Client * c)
 
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
     {
         return c->screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
     }
@@ -136,7 +139,8 @@ frameX (Client * c)
 
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
     {
         return c->x - frameLeft (c);
     }
@@ -164,7 +168,8 @@ frameWidth (Client * c)
 
     g_return_val_if_fail (c != NULL, 0);
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && !FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
     {
         return c->width + frameLeft (c) + frameRight (c);
     }
@@ -191,18 +196,49 @@ frameHeight (Client * c)
     return c->height;
 }
 
+static int
+frameTopLeftWidth (Client * c, int state)
+{
+    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    {
+        return 0;
+    }
+    return c->screen_info->corners[CORNER_TOP_LEFT][state].width;
+
+}
+
+static int
+frameTopRightWidth (Client * c, int state)
+{
+    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    {
+        return 0;
+    }
+    return c->screen_info->corners[CORNER_TOP_RIGHT][state].width;
+}
+
+static int
+frameButtonOffset (Client *c)
+{
+    if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
+    {
+        return MAX (0, c->screen_info->params->maximized_offset);
+    }
+    return c->screen_info->params->button_offset;
+}
+
 static void
 frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap * pm)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    GdkPixmap *gpixmap = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    GdkPixmap *gpixmap;
     GdkGCValues values;
-    GdkGC *gc = NULL;
-    PangoLayout *layout = NULL;
+    GdkGC *gc;
+    PangoLayout *layout;
     PangoRectangle logical_rect;
-    int width, x = 0, tp = 0, w1 = 0, w2, w3, w4, w5, temp;
-    int voffset = 0;
+    int width, x, hoffset, w1, w2, w3, w4, w5, temp;
+    int voffset, title_x, title_y;
 
     TRACE ("entering frameCreateTitlePixmap");
 
@@ -219,9 +255,7 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
         right = temp;
     }
 
-    width =
-        frameWidth (c) - screen_info->corners[CORNER_TOP_LEFT][state].width -
-        screen_info->corners[CORNER_TOP_RIGHT][state].width;
+    width = frameWidth (c) - frameTopLeftWidth (c, state) - frameTopRightWidth (c, state);
     if (width < 1)
     {
         pm->pixmap = None;
@@ -231,24 +265,27 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
         return;
     }
 
-    if (left < screen_info->corners[CORNER_TOP_LEFT][state].width)
+    if (left < frameTopLeftWidth (c, state))
     {
-        left = screen_info->corners[CORNER_TOP_LEFT][state].width;
+        left = frameTopLeftWidth (c, state);
     }
-    if (right >
-        frameWidth (c) - screen_info->corners[CORNER_TOP_RIGHT][state].width)
+    if (right > frameWidth (c) - frameTopRightWidth (c, state))
     {
-        right =
-            frameWidth (c) - screen_info->corners[CORNER_TOP_RIGHT][state].width;
+        right = frameWidth (c) - frameTopRightWidth (c, state);
     }
-    if (right < screen_info->corners[CORNER_TOP_LEFT][state].width)
+    if (right < frameTopLeftWidth (c, state))
     {
-        right = screen_info->corners[CORNER_TOP_LEFT][state].width;
+        right = frameTopLeftWidth (c, state);
     }
 
-    left = left - screen_info->corners[CORNER_TOP_LEFT][state].width;
-    right = right - screen_info->corners[CORNER_TOP_LEFT][state].width;
+    left = left - frameTopLeftWidth (c, state);
+    right = right - frameTopLeftWidth (c, state);
 
+    x = 0;
+    hoffset = 0;
+    voffset = 0;
+
+    w1 = 0;
     w2 = screen_info->title[TITLE_2][state].width;
     w4 = screen_info->title[TITLE_4][state].width;
 
@@ -267,18 +304,18 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
         switch (screen_info->params->title_alignment)
         {
             case ALIGN_LEFT:
-                tp = screen_info->params->title_horizontal_offset;
+                hoffset = screen_info->params->title_horizontal_offset;
                 break;
             case ALIGN_RIGHT:
-                tp = w3 - logical_rect.width - screen_info->params->title_horizontal_offset;
+                hoffset = w3 - logical_rect.width - screen_info->params->title_horizontal_offset;
                 break;
             case ALIGN_CENTER:
-                tp = (w3 >> 1) - (logical_rect.width >> 1);
+                hoffset = (w3 / 2) - (logical_rect.width / 2);
                 break;
         }
-        if (tp < screen_info->params->title_horizontal_offset)
+        if (hoffset < screen_info->params->title_horizontal_offset)
         {
-            tp = screen_info->params->title_horizontal_offset;
+            hoffset = screen_info->params->title_horizontal_offset;
         }
     }
     else
@@ -302,7 +339,7 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
                 w1 = right - w2 - w3 - w4 - screen_info->params->title_horizontal_offset;
                 break;
             case ALIGN_CENTER:
-                w1 = left + ((right - left) / 2) - (w3 >> 1) - w2;
+                w1 = left + ((right - left) / 2) - (w3 / 2) - w2;
                 break;
         }
         if (w1 < left)
@@ -336,18 +373,31 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
             voffset = screen_info->params->title_vertical_offset_inactive;
         }
         xfwmPixmapFill (&screen_info->title[TITLE_3][state], pm, x, 0, w3, frameTop (c)); 
+        title_x = hoffset + x;
+        title_y = voffset + (frameTop (c) - logical_rect.height) / 2;
+        if (title_y + logical_rect.height > frameTop (c))
+        {
+            title_y = MAX (0, frameTop (c) - logical_rect.height);
+        }
         if (screen_info->params->title_shadow[state])
         {
             gdk_gc_get_values (screen_info->title_shadow_colors[state].gc, &values);
             gdk_gc_set_values (gc, &values, GDK_GC_FOREGROUND);
-            gdk_draw_layout (gpixmap, gc, x + tp + 1,
-                (frameTop (c) + voffset - logical_rect.height) / 2 + 1,
-                layout);
+            if (screen_info->params->title_shadow[state] == TITLE_SHADOW_UNDER)
+            {
+                gdk_draw_layout (gpixmap, gc, title_x + 1, title_y + 1, layout);
+            }
+            else 
+            {
+                gdk_draw_layout (gpixmap, gc, title_x - 1, title_y, layout);
+                gdk_draw_layout (gpixmap, gc, title_x, title_y - 1, layout);
+                gdk_draw_layout (gpixmap, gc, title_x + 1, title_y, layout);
+                gdk_draw_layout (gpixmap, gc, title_x, title_y + 1, layout);
+            }
         }
         gdk_gc_get_values (screen_info->title_colors[state].gc, &values);
         gdk_gc_set_values (gc, &values, GDK_GC_FOREGROUND);
-        gdk_draw_layout (gpixmap, gc, x + tp,
-            (frameTop (c) + voffset - logical_rect.height) / 2, layout);
+        gdk_draw_layout (gpixmap, gc, title_x, title_y, layout);
         x = x + w3;
     }
 
@@ -370,10 +420,11 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
 static int
 getButtonFromLetter (char chr, Client * c)
 {
-    int b = -1;
+    int b;
 
     TRACE ("entering getButtonFromLetter");
 
+    b = -1;
     switch (chr)
     {
         case 'H':
@@ -421,10 +472,11 @@ getButtonFromLetter (char chr, Client * c)
 static char
 getLetterFromButton (int i, Client * c)
 {
-    char chr = 0;
+    char chr;
 
     TRACE ("entering getLetterFromButton");
 
+    chr = 0;
     switch (i)
     {
         case HIDE_BUTTON:
@@ -469,8 +521,9 @@ getLetterFromButton (int i, Client * c)
 static xfwmPixmap *
 frameGetPixmap (Client * c, int button, int state)
 {
-    ScreenInfo *screen_info = c->screen_info;
+    ScreenInfo *screen_info;
 
+    screen_info = c->screen_info;
     switch (button)
     {
         case MENU_BUTTON:
@@ -511,18 +564,21 @@ frameGetPixmap (Client * c, int button, int state)
 
 
 static void
-frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x[BUTTON_COUNT])
+frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTTON_COUNT])
 {
-    ScreenInfo *screen_info = c->screen_info;
-    DisplayInfo *display_info = screen_info->display_info;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     Window temp;
     XRectangle rect;
     xfwmPixmap *my_pixmap;
     int i;
-   
+
     TRACE ("entering frameSetShape");
     TRACE ("setting shape for client (0x%lx)", c->window);
 
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
     if (!display_info->have_shape)
     {
         return;
@@ -544,29 +600,29 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
         XShapeCombineShape (display_info->dpy, temp, ShapeBounding, frameLeft (c),
                             frameTop (c), c->window, ShapeBounding, ShapeSet);
     }
-    if (pm_cache)
+    if (frame_pix)
     {
         XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->title), ShapeBounding,
-                           0, 0, pm_cache->pm_title[state].mask, ShapeSet);
+                           0, 0, frame_pix->pm_title.mask, ShapeSet);
         if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             if (xfwmWindowVisible (&c->sides[SIDE_LEFT]))
             {
                 XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_LEFT]),
-                                   ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_LEFT][state].mask, ShapeSet);
+                                   ShapeBounding, 0, 0, frame_pix->pm_sides[SIDE_LEFT].mask, ShapeSet);
             }
 
             if (xfwmWindowVisible (&c->sides[SIDE_RIGHT]))
             {
                 XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_RIGHT]),
-                                   ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_RIGHT][state].mask, ShapeSet);
+                                   ShapeBounding, 0, 0, frame_pix->pm_sides[SIDE_RIGHT].mask, ShapeSet);
             }
         }
 
         if (xfwmWindowVisible (&c->sides[SIDE_BOTTOM]))
         {
             XShapeCombineMask (display_info->dpy, MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]),
-                               ShapeBounding, 0, 0, pm_cache->pm_sides[SIDE_BOTTOM][state].mask, ShapeSet);
+                               ShapeBounding, 0, 0, frame_pix->pm_sides[SIDE_BOTTOM].mask, ShapeSet);
         }
 
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
@@ -603,29 +659,32 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
             }
         }
 
-        if (screen_info->corners[CORNER_TOP_LEFT][state].height > frameHeight (c) - frameBottom (c) + 1)
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]) &&
+            (screen_info->corners[CORNER_TOP_LEFT][state].height > frameHeight (c) - frameBottom (c) + 1))
         {
             rect.x      = 0;
             rect.y      = frameHeight (c) - frameBottom (c) + 1;
-            rect.width  = screen_info->corners[CORNER_TOP_LEFT][state].width;
+            rect.width  = frameTopLeftWidth (c, state);
             rect.height = screen_info->corners[CORNER_TOP_LEFT][state].height
                            - (frameHeight (c) - frameBottom (c) + 1);
             XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), 
                                      ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
 
-        if (screen_info->corners[CORNER_TOP_RIGHT][state].height > frameHeight (c) - frameBottom (c) + 1)
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_RIGHT]) &&
+            (screen_info->corners[CORNER_TOP_RIGHT][state].height > frameHeight (c) - frameBottom (c) + 1))
         {
             rect.x      = 0;
             rect.y      = frameHeight (c) - frameBottom (c) + 1;
-            rect.width  = screen_info->corners[CORNER_TOP_RIGHT][state].width;
+            rect.width  = frameTopRightWidth (c, state);
             rect.height = screen_info->corners[CORNER_TOP_RIGHT][state].height
                            - (frameHeight (c) - frameBottom (c) + 1);
             XShapeCombineRectangles (display_info->dpy, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]),
                                      ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
 
-        if (screen_info->corners[CORNER_BOTTOM_LEFT][state].height > frameHeight (c) - frameTop (c) + 1)
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]) &&
+            (screen_info->corners[CORNER_BOTTOM_LEFT][state].height > frameHeight (c) - frameTop (c) + 1))
         {
             rect.x      = 0;
             rect.y      = 0;
@@ -636,7 +695,8 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
                                      ShapeBounding, 0, 0, &rect, 1, ShapeSubtract, 0);
         }
 
-        if (screen_info->corners[CORNER_BOTTOM_RIGHT][state].height > frameHeight (c) - frameTop (c) + 1)
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_RIGHT]) &&
+            (screen_info->corners[CORNER_BOTTOM_RIGHT][state].height > frameHeight (c) - frameTop (c) + 1))
         {
             rect.x      = 0;
             rect.y      = 0;
@@ -662,28 +722,35 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
             }
         }
 
-        if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]))
+        if (xfwmWindowVisible (&c->title))
         {
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-                                screen_info->corners[CORNER_TOP_LEFT][state].width, 0,
+                                frameTopLeftWidth (c, state), 0,
                                 MYWINDOW_XWINDOW (c->title), ShapeBounding, ShapeUnion);
+        }
+
+        if (xfwmWindowVisible (&c->corners[CORNER_TOP_LEFT]))
+        {
 
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0, 0,
                                 MYWINDOW_XWINDOW (c->corners[CORNER_TOP_LEFT]), ShapeBounding, ShapeUnion);
         }
-        
-        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
+
+        if (xfwmWindowVisible (&c->sides[SIDE_BOTTOM]))
         {
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
                                 screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
                                 frameHeight (c) - frameBottom (c),
                                 MYWINDOW_XWINDOW (c->sides[SIDE_BOTTOM]), ShapeBounding, ShapeUnion);
+        }
 
+        if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_LEFT]))
+        {
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding, 0,
                                 frameHeight (c) - screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
                                 MYWINDOW_XWINDOW (c->corners[CORNER_BOTTOM_LEFT]), ShapeBounding, ShapeUnion);
         }
-        
+
         if (xfwmWindowVisible (&c->corners[CORNER_BOTTOM_RIGHT]))
         {
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
@@ -695,7 +762,7 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
         if (xfwmWindowVisible (&c->corners[CORNER_TOP_RIGHT]))
         {
             XShapeCombineShape (display_info->dpy, temp, ShapeBounding,
-                                frameWidth (c) - screen_info->corners[CORNER_TOP_RIGHT][state].width,
+                                frameWidth (c) - frameTopRightWidth (c, state),
                                 0, MYWINDOW_XWINDOW (c->corners[CORNER_TOP_RIGHT]), ShapeBounding, ShapeUnion);
         }
         
@@ -710,24 +777,28 @@ frameSetShape (Client * c, int state, ClientPixmapCache * pm_cache, int button_x
         }
     }
 
+    rect.x = 0;
+    rect.y = 0;
+    rect.width  = frameWidth (c);
+    rect.height = frameHeight (c);
+    XShapeCombineRectangles (display_info->dpy, temp, ShapeBounding, 0, 0, &rect, 1, ShapeIntersect, 0);
     XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, temp, ShapeBounding, ShapeSet);
 
     XDestroyWindow (display_info->dpy, temp);
 }
 
 void
-frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
+frameDraw (Client * c, gboolean clear_all)
 {
-    ScreenInfo *screen_info = NULL;
+    ScreenInfo *screen_info;
+    FramePixmap frame_pix;
     xfwmPixmap *my_pixmap;
-    gboolean requires_clearing = FALSE;
-    gboolean width_changed = FALSE;
-    gboolean height_changed = FALSE;
-    gboolean title_changed = FALSE;
-    int state = ACTIVE;
-    int i, j, x, button, left, right;
+    int state, i, j, x, button, left, right;
     int top_width, bottom_width, left_height, right_height;
     int button_x[BUTTON_COUNT];
+    gboolean requires_clearing;
+    gboolean width_changed;
+    gboolean height_changed;
 
     TRACE ("entering frameDraw");
     TRACE ("drawing frame for \"%s\" (0x%lx)", c->name, c->window);
@@ -735,6 +806,11 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
     g_return_if_fail (c != NULL);
 
     screen_info = c->screen_info;
+    requires_clearing = FALSE;
+    width_changed = FALSE;
+    height_changed = FALSE;
+    state = ACTIVE;
+
     if (c != clientGetFocus ())
     {
         TRACE ("\"%s\" is not the active window", c->name);
@@ -772,44 +848,26 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
     FLAG_UNSET (c->xfwm_flags,  XFWM_FLAG_FIRST_MAP);
 
     /* Cache mgmt */
-    if (invalidate_cache)
+    if (clear_all)
     {
-        clientClearPixmapCache (c);
+        width_changed = TRUE;
+        height_changed = TRUE;
         requires_clearing = TRUE;
     }
     else
     {
-        if (c->pm_cache.previous_width != c->width)
+        if (c->previous_width != c->width)
         {
-            xfwmPixmapFree (&c->pm_cache.pm_title[ACTIVE]);
-            xfwmPixmapFree (&c->pm_cache.pm_title[INACTIVE]);
-            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][ACTIVE]);
-            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_BOTTOM][INACTIVE]);
-            c->pm_cache.previous_width = c->width;
+            width_changed = TRUE;
+            c->previous_width = c->width;
         }
-        if (c->pm_cache.previous_height != c->height)
+        if (c->previous_height != c->height)
         {
-            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_LEFT][ACTIVE]);
-            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_LEFT][INACTIVE]);
-            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][ACTIVE]);
-            xfwmPixmapFree (&c->pm_cache.pm_sides[SIDE_RIGHT][INACTIVE]);
-            c->pm_cache.previous_height = c->height;
+            height_changed = TRUE;
+            c->previous_height = c->height;
         }
     }
 
-    /* Corners have no caching and get never resized, we need to update them separately */
-    if (requires_clearing)
-    {
-        xfwmWindowSetBG (&c->corners[CORNER_TOP_LEFT], 
-                         &screen_info->corners[CORNER_TOP_LEFT][state]);
-        xfwmWindowSetBG (&c->corners[CORNER_TOP_RIGHT], 
-                         &screen_info->corners[CORNER_TOP_RIGHT][state]);
-        xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_LEFT], 
-                         &screen_info->corners[CORNER_BOTTOM_LEFT][state]);
-        xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_RIGHT], 
-                         &screen_info->corners[CORNER_BOTTOM_RIGHT][state]);
-    }
-
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
     {
@@ -824,8 +882,12 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         }
 
         /* Then, show the ones that we do have on left... */
-        x = frameLeft (c) + screen_info->params->button_offset;
-        right = frameWidth (c) - frameRight (c) - screen_info->params->button_offset;
+        x = frameLeft (c) + frameButtonOffset (c);
+        if (x < 0)
+        {
+            x = 0;
+        }
+        right = frameWidth (c) - frameRight (c) - frameButtonOffset (c);
         for (i = 0; i < strlen (screen_info->params->button_layout); i++)
         {
             button = getButtonFromLetter (screen_info->params->button_layout[i], c);
@@ -860,7 +922,7 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
 
         /* and those that we do have on right... */
         x = frameWidth (c) - frameRight (c) + screen_info->params->button_spacing -
-            screen_info->params->button_offset;
+            frameButtonOffset (c);
         for (j = strlen (screen_info->params->button_layout) - 1; j >= i; j--)
         {
             button = getButtonFromLetter (screen_info->params->button_layout[j], c);
@@ -894,124 +956,125 @@ frameDraw (Client * c, gboolean invalidate_cache, gboolean force_shape_update)
         left = left - 2 * screen_info->params->button_spacing;
         right = x;
 
-        top_width =
-            frameWidth (c) - screen_info->corners[CORNER_TOP_LEFT][state].width -
-            screen_info->corners[CORNER_TOP_RIGHT][state].width;
-        bottom_width =
-            frameWidth (c) -
+        top_width = frameWidth (c) - frameTopLeftWidth (c, state) - frameTopRightWidth (c, state);
+        bottom_width = frameWidth (c) -
             screen_info->corners[CORNER_BOTTOM_LEFT][state].width -
             screen_info->corners[CORNER_BOTTOM_RIGHT][state].width;
-        left_height =
-            frameHeight (c) - frameTop (c) -
+        left_height = frameHeight (c) - frameTop (c) -
             screen_info->corners[CORNER_BOTTOM_LEFT][state].height;
-        right_height =
-            frameHeight (c) - frameTop (c) -
+        right_height = frameHeight (c) - frameTop (c) -
             screen_info->corners[CORNER_BOTTOM_RIGHT][state].height;
 
-        if (c->pm_cache.pm_title[state].pixmap == None)
+        xfwmPixmapInit (screen_info, &frame_pix.pm_title);
+        xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_LEFT]);
+        xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_RIGHT]);
+        xfwmPixmapInit (screen_info, &frame_pix.pm_sides[SIDE_BOTTOM]);
+
+        /* The title is always visible */
+        frameCreateTitlePixmap (c, state, left, right, &frame_pix.pm_title);
+        xfwmWindowSetBG (&c->title, &frame_pix.pm_title);
+        xfwmWindowShow (&c->title,
+            frameTopLeftWidth (c, state), 0, top_width,
+            frameTop (c), (requires_clearing | width_changed));
+
+        /* Corners are never resized, we need to update them separately */
+        if (requires_clearing)
         {
-            frameCreateTitlePixmap (c, state, left, right,
-                &c->pm_cache.pm_title[state]);
-            title_changed = TRUE;
+            xfwmWindowSetBG (&c->corners[CORNER_TOP_LEFT], 
+                &screen_info->corners[CORNER_TOP_LEFT][state]);
+            xfwmWindowSetBG (&c->corners[CORNER_TOP_RIGHT], 
+                &screen_info->corners[CORNER_TOP_RIGHT][state]);
+            xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_LEFT], 
+                &screen_info->corners[CORNER_BOTTOM_LEFT][state]);
+            xfwmWindowSetBG (&c->corners[CORNER_BOTTOM_RIGHT], 
+                &screen_info->corners[CORNER_BOTTOM_RIGHT][state]);
         }
 
-        if (c->pm_cache.pm_sides[SIDE_LEFT][state].pixmap == None)
-        {
-            xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_LEFT][state],
-                frameLeft (c), left_height);
-            height_changed = TRUE;
-        }
-        xfwmPixmapFill (&screen_info->sides[SIDE_LEFT][state], 
-                        &c->pm_cache.pm_sides[SIDE_LEFT][state],
-                        0, 0, frameLeft (c), left_height); 
-        if (c->pm_cache.pm_sides[SIDE_RIGHT][state].pixmap == None)
-        {
-            xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_RIGHT][state],
-                frameRight (c), right_height);
-            height_changed = TRUE;
-        }
-        xfwmPixmapFill (&screen_info->sides[SIDE_RIGHT][state], 
-                        &c->pm_cache.pm_sides[SIDE_RIGHT][state], 
-                        0, 0, frameRight (c), right_height); 
-        if (c->pm_cache.pm_sides[SIDE_BOTTOM][state].pixmap == None)
-        {
-            xfwmPixmapCreate (screen_info, &c->pm_cache.pm_sides[SIDE_BOTTOM][state],
-                bottom_width, frameBottom (c));
-            width_changed = TRUE;
-        }
-        xfwmPixmapFill (&screen_info->sides[SIDE_BOTTOM][state], 
-                        &c->pm_cache.pm_sides[SIDE_BOTTOM][state], 
-                        0, 0, bottom_width, frameBottom (c));
-
-        if (requires_clearing || title_changed)
-        {
-            xfwmWindowSetBG (&c->title, 
-                             &c->pm_cache.pm_title[state]);
-        }
-
-        if (requires_clearing || width_changed)
-        {
-            xfwmWindowSetBG (&c->sides[SIDE_BOTTOM], 
-                             &c->pm_cache.pm_sides[SIDE_BOTTOM][state]);
-        }
-
-        if (requires_clearing || height_changed)
-        {
-            xfwmWindowSetBG (&c->sides[SIDE_LEFT], 
-                             &c->pm_cache.pm_sides[SIDE_LEFT][state]);
-            xfwmWindowSetBG (&c->sides[SIDE_RIGHT], 
-                             &c->pm_cache.pm_sides[SIDE_RIGHT][state]);
-        }
-
-        if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
+        if (FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED))
         {
             xfwmWindowHide (&c->sides[SIDE_LEFT]);
             xfwmWindowHide (&c->sides[SIDE_RIGHT]);
+            xfwmWindowHide (&c->sides[SIDE_BOTTOM]);
+            xfwmWindowHide (&c->corners[CORNER_TOP_LEFT]);
+            xfwmWindowHide (&c->corners[CORNER_TOP_RIGHT]);
+            xfwmWindowHide (&c->corners[CORNER_BOTTOM_LEFT]);
+            xfwmWindowHide (&c->corners[CORNER_BOTTOM_RIGHT]);
         }
         else
         {
-            xfwmWindowShow (&c->sides[SIDE_LEFT], 0, frameTop (c),
-                frameLeft (c), left_height, (requires_clearing | height_changed));
-            xfwmWindowShow (&c->sides[SIDE_RIGHT],
-                frameWidth (c) - frameRight (c), frameTop (c), frameRight (c),
-                right_height, (requires_clearing | height_changed));
-        }
+            if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
+            {
+                xfwmWindowHide (&c->sides[SIDE_LEFT]);
+                xfwmWindowHide (&c->sides[SIDE_RIGHT]);
+            }
+            else
+            {
+                xfwmPixmapCreate (screen_info, &frame_pix.pm_sides[SIDE_LEFT],
+                    frameLeft (c), left_height);
+                xfwmPixmapFill (&screen_info->sides[SIDE_LEFT][state], 
+                    &frame_pix.pm_sides[SIDE_LEFT],
+                    0, 0, frameLeft (c), left_height); 
+                xfwmWindowSetBG (&c->sides[SIDE_LEFT], 
+                    &frame_pix.pm_sides[SIDE_LEFT]);
+                xfwmWindowShow (&c->sides[SIDE_LEFT], 0, frameTop (c),
+                    frameLeft (c), left_height, (requires_clearing | height_changed));
 
-        xfwmWindowShow (&c->title,
-            screen_info->corners[CORNER_TOP_LEFT][state].width, 0, top_width,
-            frameTop (c), (requires_clearing | title_changed));
-        xfwmWindowShow (&c->sides[SIDE_BOTTOM],
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
-            frameHeight (c) - frameBottom (c), bottom_width, frameBottom (c),
-            (requires_clearing | width_changed));
-        xfwmWindowShow (&c->corners[CORNER_TOP_LEFT], 0, 0,
-            screen_info->corners[CORNER_TOP_LEFT][state].width,
-            screen_info->corners[CORNER_TOP_LEFT][state].height,
-            requires_clearing);
-        xfwmWindowShow (&c->corners[CORNER_TOP_RIGHT],
-            frameWidth (c) - screen_info->corners[CORNER_TOP_RIGHT][state].width,
-            0, screen_info->corners[CORNER_TOP_RIGHT][state].width,
-            screen_info->corners[CORNER_TOP_RIGHT][state].height,
-            requires_clearing);
-        xfwmWindowShow (&c->corners[CORNER_BOTTOM_LEFT], 0,
-            frameHeight (c) -
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
-            screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
-            requires_clearing);
-        xfwmWindowShow (&c->corners[CORNER_BOTTOM_RIGHT],
-            frameWidth (c) -
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
-            frameHeight (c) -
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
-            screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
-            requires_clearing);
+                xfwmPixmapCreate (screen_info, &frame_pix.pm_sides[SIDE_RIGHT],
+                    frameRight (c), right_height);
+                xfwmPixmapFill (&screen_info->sides[SIDE_RIGHT][state], 
+                    &frame_pix.pm_sides[SIDE_RIGHT], 
+                    0, 0, frameRight (c), right_height); 
+                xfwmWindowSetBG (&c->sides[SIDE_RIGHT], 
+                    &frame_pix.pm_sides[SIDE_RIGHT]);
+                xfwmWindowShow (&c->sides[SIDE_RIGHT],
+                    frameWidth (c) - frameRight (c), frameTop (c), frameRight (c),
+                    right_height, (requires_clearing | height_changed));
+            }
 
-        if (requires_clearing | force_shape_update)
-        {
-            frameSetShape (c, state, &c->pm_cache, button_x);
+            xfwmPixmapCreate (screen_info, &frame_pix.pm_sides[SIDE_BOTTOM],
+                bottom_width, frameBottom (c));
+            xfwmPixmapFill (&screen_info->sides[SIDE_BOTTOM][state], 
+                &frame_pix.pm_sides[SIDE_BOTTOM], 
+                0, 0, bottom_width, frameBottom (c));
+            xfwmWindowSetBG (&c->sides[SIDE_BOTTOM], 
+                &frame_pix.pm_sides[SIDE_BOTTOM]);
+            xfwmWindowShow (&c->sides[SIDE_BOTTOM],
+                screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
+                frameHeight (c) - frameBottom (c), bottom_width, frameBottom (c),
+                (requires_clearing | width_changed));
+
+            xfwmWindowShow (&c->corners[CORNER_TOP_LEFT], 0, 0,
+                frameTopLeftWidth (c, state),
+                screen_info->corners[CORNER_TOP_LEFT][state].height,
+                requires_clearing);
+
+            xfwmWindowShow (&c->corners[CORNER_TOP_RIGHT],
+                frameWidth (c) - frameTopRightWidth (c, state),
+                0, frameTopRightWidth (c, state),
+                screen_info->corners[CORNER_TOP_RIGHT][state].height,
+                requires_clearing);
+
+            xfwmWindowShow (&c->corners[CORNER_BOTTOM_LEFT], 0,
+                frameHeight (c) -
+                screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
+                screen_info->corners[CORNER_BOTTOM_LEFT][state].width,
+                screen_info->corners[CORNER_BOTTOM_LEFT][state].height,
+                requires_clearing);
+
+            xfwmWindowShow (&c->corners[CORNER_BOTTOM_RIGHT],
+                frameWidth (c) -
+                screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
+                frameHeight (c) -
+                screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
+                screen_info->corners[CORNER_BOTTOM_RIGHT][state].width,
+                screen_info->corners[CORNER_BOTTOM_RIGHT][state].height,
+                requires_clearing);
         }
+        frameSetShape (c, state, &frame_pix, button_x);
+        xfwmPixmapFree (&frame_pix.pm_title);
+        xfwmPixmapFree (&frame_pix.pm_sides[SIDE_BOTTOM]);
+        xfwmPixmapFree (&frame_pix.pm_sides[SIDE_LEFT]);
+        xfwmPixmapFree (&frame_pix.pm_sides[SIDE_RIGHT]);
     }
     else
     {
