@@ -1,8 +1,9 @@
-/*
+/*      $Id: frame.h 22847 2006-08-21 20:13:05Z olivier $
+ 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2003 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -25,8 +26,21 @@
 #  include "config.h"
 #endif
 
+#include "screen.h"
+#include "mypixmap.h"
 #include "client.h"
 
+struct _FramePixmap
+{
+    xfwmPixmap pm_title;
+    xfwmPixmap pm_sides[3];
+};
+typedef struct _FramePixmap FramePixmap;
+
+int frameDecorationLeft (ScreenInfo *);
+int frameDecorationRight (ScreenInfo *);
+int frameDecorationTop (ScreenInfo *);
+int frameDecorationBottom (ScreenInfo *);
 int frameLeft (Client *);
 int frameRight (Client *);
 int frameTop (Client *);
@@ -35,6 +49,6 @@ int frameX (Client *);
 int frameY (Client *);
 int frameWidth (Client *);
 int frameHeight (Client *);
-void frameDraw (Client *, gboolean, gboolean);
+void frameDraw (Client *, gboolean);
 
 #endif /* INC_FRAME_H */
