@@ -1,4 +1,4 @@
-/*      $Id: hints.h,v 1.55 2004/11/23 23:57:33 olivier Exp $
+/*      $Id: hints.h 22955 2006-08-30 19:53:33Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -37,6 +37,7 @@
 #include "display.h"
 
 #define MWM_HINTS_ELEMENTS                      3L
+#define MAX_STR_LENGTH                          255
 
 #define MWM_HINTS_FUNCTIONS                     (1L << 0)
 #define MWM_HINTS_DECORATIONS                   (1L << 1)
@@ -69,7 +70,8 @@
 #define WIN_STATE_STICKY                        (1L<<0)
 #define WIN_STATE_MAXIMIZED_VERT                (1L<<2)
 #define WIN_STATE_MAXIMIZED_HORIZ               (1L<<3)
-#define WIN_STATE_MAXIMIZED                     ((1L<<2)|(1L<<3))
+#define WIN_STATE_MAXIMIZED                     (WIN_STATE_MAXIMIZED_VERT | \
+                                                 WIN_STATE_MAXIMIZED_HORIZ)
 #define WIN_STATE_SHADED                        (1L<<5)
 
 #define WIN_HINTS_SKIP_FOCUS                    (1L<<0)
@@ -110,6 +112,11 @@
 
 #define NET_WM_OPAQUE                           0xffffffff
 
+/* Convenient macro */
+#define HINTS_ACCEPT_INPUT(wmhints)     (!(wmhints) ||                                                              \
+                                         ((wmhints) && !(wmhints->flags & InputHint)) ||                            \
+                                         ((wmhints) && (wmhints->flags & InputHint) && (wmhints->input)))
+
 typedef struct
 {
     unsigned long orientation;
@@ -155,22 +162,30 @@ gboolean getCardinalList (DisplayInfo *, Window, int, unsigned long **, int *);
 void setNetWorkarea (DisplayInfo *, Window, int, int, int, int *);
 void setNetFrameExtents (DisplayInfo *, Window, int, int, int, int);
 void initNetDesktopInfo (DisplayInfo *, Window, int, int, int);
-void setUTF8StringHint (DisplayInfo *, Window, int, const char *);
+void setUTF8StringHint (DisplayInfo *, Window, int, const gchar *);
 void getTransientFor (DisplayInfo *, Window, Window, Window *);
-void getWindowName (DisplayInfo *, Window, char **);
-gboolean getUTF8String (DisplayInfo *, Window, int, char **, int *);
-void getWindowName (DisplayInfo *, Window, char **);
-gboolean checkKdeSystrayWindow(DisplayInfo *, Window);
-void sendSystrayReqDock(DisplayInfo *, Window, Window);
-Window getSystrayWindow (DisplayInfo *, Atom);
-gboolean getWindowRole (DisplayInfo *, Window, char **);
+gboolean getWindowName (DisplayInfo *, Window, gchar **);
+gboolean getUTF8String (DisplayInfo *, Window, int, gchar **, int *);
+gboolean getUTF8StringList (DisplayInfo *, Window, int, gchar ***, int *);
+gboolean getClientMachine (DisplayInfo *, Window, gchar **);
+gboolean getClientMachine (DisplayInfo *, Window, gchar **);
+gboolean getWindowRole (DisplayInfo *, Window, gchar **);
 Window getClientLeader (DisplayInfo *, Window);
 gboolean getNetWMUserTime (DisplayInfo *, Window, Time *);
-gboolean getClientID (DisplayInfo *, Window, char **);
+gboolean getClientID (DisplayInfo *, Window, gchar **);
 gboolean getWindowCommand (DisplayInfo *, Window, char ***, int *);
 gboolean getKDEIcon (DisplayInfo *, Window, Pixmap *, Pixmap *);
 gboolean getRGBIconData (DisplayInfo *, Window, unsigned long **, unsigned long *);
 gboolean getOpacity (DisplayInfo *, Window, guint *);
+gboolean getOpacityLock (DisplayInfo *, Window);
+gboolean setXAtomManagerOwner (DisplayInfo *, Atom, Window , Window);
+gboolean setAtomIdManagerOwner (DisplayInfo *, int, Window , Window);
+
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
+gboolean checkKdeSystrayWindow(DisplayInfo *, Window);
+void sendSystrayReqDock(DisplayInfo *, Window, Window);
+Window getSystrayWindow (DisplayInfo *, Atom);
+#endif
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 gboolean getWindowStartupId (DisplayInfo *, Window, char **);
