@@ -1,4 +1,4 @@
-/*      $Id: keyboard.c 12290 2005-01-16 21:21:19Z olivier $
+/*      $Id: keyboard.c 22470 2006-07-17 17:09:25Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -31,9 +31,6 @@
 #include <libxfce4util/libxfce4util.h> 
 #include "keyboard.h"
 
-unsigned int KeyMask;
-unsigned int ButtonMask;
-unsigned int ButtonKeyMask;
 unsigned int AltMask;
 unsigned int MetaMask;
 unsigned int NumLockMask;
@@ -41,6 +38,64 @@ unsigned int ScrollLockMask;
 unsigned int SuperMask;
 unsigned int HyperMask;
 
+int
+getModifierMap (char *str)
+{
+    gchar *tmp;
+    int map;
+
+    tmp = g_ascii_strdown ((gchar *) str, -1);
+    map = 0;
+
+    if (strstr (tmp, "shift"))
+    {
+        map |= ShiftMask;
+    }
+    if (strstr (tmp, "control"))
+    {
+        map |= ControlMask;
+    }
+    if (strstr (tmp, "alt"))
+    {
+        map |= AltMask;
+    }
+    if (strstr (tmp, "meta"))
+    {
+        map |= MetaMask;
+    }
+    if (strstr (tmp, "super"))
+    {
+        map |= SuperMask;
+    }
+    if (strstr (tmp, "hyper"))
+    {
+        map |= HyperMask;
+    }
+    if (strstr (tmp, "mod1"))
+    {
+        map |= Mod1Mask;
+    }
+    if (strstr (tmp, "mod2"))
+    {
+        map |= Mod2Mask;
+    }
+    if (strstr (tmp, "mod3"))
+    {
+        map |= Mod3Mask;
+    }
+    if (strstr (tmp, "mod4"))
+    {
+        map |= Mod4Mask;
+    }
+    if (strstr (tmp, "mod5"))
+    {
+        map |= Mod5Mask;
+    }
+    g_free (tmp);
+
+    return map;
+}
+
 void
 parseKeyString (Display * dpy, MyKey * key, char *str)
 {
@@ -65,56 +120,8 @@ parseKeyString (Display * dpy, MyKey * key, char *str)
     if (k)
     {
         /* There is a modifier */
-        gchar *tmp;
-
-        tmp = g_ascii_strdown ((gchar *) str, -1);
-
         key->keycode = XKeysymToKeycode (dpy, XStringToKeysym (++k));
-        if (strstr (tmp, "shift"))
-        {
-            key->modifier = key->modifier | ShiftMask;
-        }
-        if (strstr (tmp, "control"))
-        {
-            key->modifier = key->modifier | ControlMask;
-        }
-        if (strstr (tmp, "alt"))
-        {
-            key->modifier = key->modifier | AltMask;
-        }
-        if (strstr (tmp, "meta"))
-        {
-            key->modifier = key->modifier | MetaMask;
-        }
-        if (strstr (tmp, "super"))
-        {
-            key->modifier = key->modifier | SuperMask;
-        }
-        if (strstr (tmp, "hyper"))
-        {
-            key->modifier = key->modifier | HyperMask;
-        }
-        if (strstr (tmp, "mod1"))
-        {
-            key->modifier = key->modifier | Mod1Mask;
-        }
-        if (strstr (tmp, "mod2"))
-        {
-            key->modifier = key->modifier | Mod2Mask;
-        }
-        if (strstr (tmp, "mod3"))
-        {
-            key->modifier = key->modifier | Mod3Mask;
-        }
-        if (strstr (tmp, "mod4"))
-        {
-            key->modifier = key->modifier | Mod4Mask;
-        }
-        if (strstr (tmp, "mod5"))
-        {
-            key->modifier = key->modifier | Mod5Mask;
-        }
-        g_free (tmp);
+        key->modifier = getModifierMap (str);
     }
     else
     {
@@ -133,31 +140,35 @@ grabKey (Display * dpy, MyKey * key, Window w)
         if (key->modifier == 0)
         {
             XGrabKey (dpy, key->keycode, AnyModifier, w, FALSE,
-                GrabModeAsync, GrabModeAsync);
+                GrabModeAsync, GrabModeSync);
         }
         else
         {
             /* Here we grab all combinations of well known modifiers */
-            XGrabKey (dpy, key->keycode, key->modifier, w, FALSE,
-                GrabModeAsync, GrabModeAsync);
-            XGrabKey (dpy, key->keycode, key->modifier | ScrollLockMask, w,
-                FALSE, GrabModeAsync, GrabModeAsync);
-            XGrabKey (dpy, key->keycode, key->modifier | NumLockMask, w,
-                FALSE, GrabModeAsync, GrabModeAsync);
-            XGrabKey (dpy, key->keycode, key->modifier | LockMask, w,
-                FALSE, GrabModeAsync, GrabModeAsync);
+            XGrabKey (dpy, key->keycode, 
+                key->modifier, w, FALSE,
+                GrabModeAsync, GrabModeSync);
+            XGrabKey (dpy, key->keycode, 
+                key->modifier | ScrollLockMask, w,
+                FALSE, GrabModeAsync, GrabModeSync);
+            XGrabKey (dpy, key->keycode, 
+                key->modifier | NumLockMask, w,
+                FALSE, GrabModeAsync, GrabModeSync);
+            XGrabKey (dpy, key->keycode, 
+                key->modifier | LockMask, w,
+                FALSE, GrabModeAsync, GrabModeSync);
             XGrabKey (dpy, key->keycode,
                 key->modifier | ScrollLockMask | NumLockMask, w, FALSE,
-                GrabModeAsync, GrabModeAsync);
+                GrabModeAsync, GrabModeSync);
             XGrabKey (dpy, key->keycode,
                 key->modifier | ScrollLockMask | LockMask, w, FALSE,
-                GrabModeAsync, GrabModeAsync);
+                GrabModeAsync, GrabModeSync);
             XGrabKey (dpy, key->keycode,
                 key->modifier | LockMask | NumLockMask, w, FALSE,
-                GrabModeAsync, GrabModeAsync);
+                GrabModeAsync, GrabModeSync);
             XGrabKey (dpy, key->keycode,
                 key->modifier | ScrollLockMask | LockMask | NumLockMask,
-                w, FALSE, GrabModeAsync, GrabModeAsync);
+                w, FALSE, GrabModeAsync, GrabModeSync);
         }
     }
 }
@@ -249,10 +260,11 @@ initModifiers (Display * dpy)
 {
     XModifierKeymap *modmap;
     KeySym *keymap;
-    int i;
+    unsigned int keycode;
     int keysyms_per_keycode;
     int min_keycode;
     int max_keycode;
+    int i;
 
     AltMask = 0;
     MetaMask = 0;
@@ -270,8 +282,7 @@ initModifiers (Display * dpy)
 
     for (i = 3 * modmap->max_keypermod; i < 8 * modmap->max_keypermod; i++)
     {
-        unsigned int keycode = modmap->modifiermap[i];
-
+        keycode = modmap->modifiermap[i];
         if ((keycode >= min_keycode) && (keycode <= max_keycode))
         {
             int j;
@@ -306,11 +317,4 @@ initModifiers (Display * dpy)
             }
         }
     }
-    KeyMask =
-        ControlMask | ShiftMask | AltMask | MetaMask | SuperMask | HyperMask;
-
-    ButtonMask =
-        Button1Mask | Button2Mask | Button3Mask | Button4Mask | Button5Mask;
-
-    ButtonKeyMask = KeyMask | ButtonMask;
 }
