@@ -1,4 +1,4 @@
-/*      $Id: main.c 18667 2005-11-05 15:10:05Z olivier $
+/*      $Id: main.c 22954 2006-08-30 19:41:26Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -35,6 +35,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <time.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -45,6 +46,7 @@
 #include "display.h"
 #include "screen.h"
 #include "events.h"
+#include "event_filter.h"
 #include "frame.h"
 #include "settings.h"
 #include "client.h"
@@ -73,10 +75,13 @@
 #else /* HAVE_COMPOSITOR */
 #define MAIN_EVENT_MASK BASE_EVENT_MASK
 #endif /* HAVE_COMPOSITOR */
-    
-static DisplayInfo *display_info = NULL;
-gboolean xfwm4_quit           = FALSE;
-gboolean xfwm4_reload         = FALSE;
+
+#ifndef DEBUG
+/* For what, IEEE Std 1003.1-2001, Section 12.2, Utility Syntax Guidelines.*/
+static char revision[]="@(#)$ " PACKAGE " version " VERSION " revision " REVISION " $";
+#endif
+
+static DisplayInfo *display_info;
 
 static void
 cleanUp (void)
@@ -87,7 +92,7 @@ cleanUp (void)
     
     TRACE ("entering cleanUp");
 
-    xfce_close_event_filter (display_info->xfilter);
+    eventFilterClose (display_info->xfilter);
     for (screens = display_info->screens; screens; screens = g_slist_next (screens))
     {
         ScreenInfo *screen_info_n = (ScreenInfo *) screens->data;
@@ -102,64 +107,6 @@ cleanUp (void)
     display_info = NULL;
 }
 
-static char *
-build_session_filename(SessionClient *client_session)
-{
-    gchar *filename, *path, *file;
-    GError *error = NULL;
-
-    path = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
-    
-    if (!xfce_mkdirhier(path, 0700, &error)) 
-    {
-        g_warning("Unable to create session dir %s: %s", path, error->message);
-        g_error_free (error);
-        g_free (path);
-        return NULL;
-    }
-
-    file = g_strdup_printf("xfwm4-%s", client_session->given_client_id);
-    filename = g_build_filename (path, file, NULL);
-    g_free (file);
-    g_free (path);
-    
-    return filename;
-}
-
-static void
-load_saved_session (SessionClient *client_session)
-{
-    gchar *filename;
-    
-    filename = build_session_filename(client_session);
-    if (filename)
-    {
-        sessionLoadWindowStates (filename);
-        g_free (filename);
-    }
-}
-
-static void
-save_phase_2 (gpointer data)
-{
-    SessionClient *client_session = (SessionClient *) data;
-    gchar *filename;
-
-    filename = build_session_filename(client_session);
-    if (filename)
-    {
-        sessionSaveWindowStates (display_info, filename);
-        g_free (filename);
-    }
-}
-
-static void
-session_die (gpointer client_data)
-{
-    gtk_main_quit ();
-    xfwm4_quit = TRUE;
-}
-
 static void
 handleSignal (int sig)
 {
@@ -170,11 +117,11 @@ handleSignal (int sig)
         case SIGINT:
         case SIGTERM:
             gtk_main_quit ();
-            xfwm4_quit = TRUE;
+            display_info->quit = TRUE;
             break;
         case SIGHUP:
         case SIGUSR1:
-            xfwm4_reload = TRUE;
+            display_info->reload = TRUE;
             break;
         case SIGSEGV:
             cleanUp ();
@@ -189,7 +136,7 @@ static void
 ensure_basedir_spec (void)
 {
     char *new, *old, path[PATH_MAX];
-    GError *error = NULL;
+    GError *error;
     GDir *gdir;
     const char *name;
 
@@ -205,6 +152,7 @@ ensure_basedir_spec (void)
         return;
     }
 
+    error = NULL;
     if (!xfce_mkdirhier(new, 0700, &error)) 
     {
         g_warning("Unable to create config dir %s: %s", new, error->message);
@@ -233,7 +181,7 @@ ensure_basedir_spec (void)
         
         if (w && r)
         {
-            char c;
+            int c;
             
             while ((c = getc (r)) != EOF)
             {
@@ -257,7 +205,6 @@ ensure_basedir_spec (void)
     /* copy saved session data */
     
     new = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
-
     if (!xfce_mkdirhier(new, 0700, &error)) 
     {
         g_warning("Unable to create session dir %s: %s", new, error->message);
@@ -283,7 +230,7 @@ ensure_basedir_spec (void)
 
             if (w && r)
             {
-                char c;
+                int c;
                 
                 while ((c = getc (r)) != EOF)
                     putc (c, w);
@@ -305,45 +252,120 @@ ensure_basedir_spec (void)
 static void
 print_usage (void)
 {
-    g_print ("%s [--sm-client-id=ID] [--display=DISPLAY] \
-[--compositor=off|on|auto] [--daemon] \
-[--version|-V] [--help|-H]\n", PACKAGE);
-    exit (0);
+    g_print ("%s [--sm-client-id=ID] [--display=DISPLAY] "
+#ifdef HAVE_COMPOSITOR
+             "[--compositor=off|on|auto] "
+#endif
+             "[--daemon] [--version|-V] [--help|-H]\n", PACKAGE);
 }
 
 static void
 print_version (void)
 {
-    g_print ("\tThis is %s version %s for Xfce %s\n", 
-                    PACKAGE, VERSION, xfce_version_string());
-    g_print ("\tbuilt with GTK+-%d.%d.%d, ", 
+    g_print ("\tThis is %s version %s (revision %s) for Xfce %s\n", 
+                    PACKAGE, VERSION, REVISION, xfce_version_string());
+    g_print ("\tReleased under the terms of the GNU General Public License.\n"); 
+    g_print ("\tCompiled against GTK+-%d.%d.%d, ", 
                     GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
-    g_print ("linked with GTK+-%d.%d.%d.\n", 
+    g_print ("using GTK+-%d.%d.%d.\n", 
                     gtk_major_version, gtk_minor_version, gtk_micro_version);
-    exit (0);
+    g_print ("\n");
+    g_print ("\tBuild configuration and supported features:\n");
+
+    g_print ("\t- Startup notification support:                 ");
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+    g_print ("Yes\n");
+#else
+    g_print ("No\n");
+#endif    
+
+    g_print ("\t- Render support:                               ");
+#ifdef HAVE_RENDER
+    g_print ("Yes\n");
+#else
+    g_print ("No\n");
+#endif    
+
+    g_print ("\t- Xrandr support:                               ");
+#ifdef HAVE_RANDR
+    g_print ("Yes\n");
+#else
+    g_print ("No\n");
+#endif    
+
+    g_print ("\t- Embedded compositor:                          ");
+#ifdef HAVE_COMPOSITOR
+    g_print ("Yes\n");
+#else
+    g_print ("No\n");
+#endif    
+    g_print ("\t- KDE systray proxy (deprecated):               ");
+#ifdef ENABLE_KDE_SYSTRAY_PROXY
+    g_print ("Yes\n");
+#else
+    g_print ("No\n");
+#endif    
 }
 
+#ifdef HAVE_COMPOSITOR
+static gint
+get_default_compositor (DisplayInfo *display_info)
+{
+    /* 
+     * Don't even check for the render speed if there is no compositor.
+     */
+    if (!display_info->enable_compositor)
+    {
+        return 0;
+    }
+
+    /* 
+     * Check if the XServer is black listed.
+     */
+    if (!compositorTestServer (display_info))
+    {
+        g_warning ("The XServer currently in use on this system is not suitable for the compositor");
+        return 0;
+    }
+
+#if 0
+    /* 
+     * Test if the XRender implementation is fast enough for the
+     * compositor.
+     */
+    if (!myDisplayTestXrender (display_info, 0.025))
+    {
+        g_warning ("The XRender implementation currently in use on this system is too slow for the compositor");
+        return 0;
+    }
+#endif
+    return 2;
+}
+#endif /* HAVE_COMPOSITOR */
+
+#ifdef HAVE_COMPOSITOR
 static gint
 parse_compositor (const gchar *s)
 {
     gchar *rvalue;
-    gboolean retval = 2;
+    gint retval;
 
+    retval = 2;
     rvalue = strrchr (s, '=');
     if (rvalue)
     {
         rvalue++;
         if (!strcmp (rvalue, "off"))
         {
-            retval=0;
+            retval = 0;
         }
         else if (!strcmp (rvalue, "auto"))
         {
-            retval=1;
+            retval = 1;
         }
         else if (!strcmp (rvalue, "on"))
         {
-            retval=2;
+            retval = 2;
         }
         else
         {
@@ -353,13 +375,13 @@ parse_compositor (const gchar *s)
 
     return retval;
 }
+#endif /* HAVE_COMPOSITOR */
 
 static int
 initialize (int argc, char **argv, gint compositor_mode)
 {
     struct sigaction act;
     long ws;
-    SessionClient *client_session;
     gint i, nscreens;
     
     TRACE ("entering initialize");
@@ -378,11 +400,21 @@ initialize (int argc, char **argv, gint compositor_mode)
 
     display_info = myDisplayInit (gdk_display_get_default ());
     
+#ifdef HAVE_COMPOSITOR
+    if (compositor_mode < 0)
+    {
+        compositor_mode = get_default_compositor (display_info);
+    }
+    
     /* Disabling compositor from command line */
     if (!compositor_mode)
     {
         display_info->enable_compositor = FALSE;
     }
+    compositorSetCompositeMode (display_info, (compositor_mode == 2));
+#else /* HAVE_COMPOSITOR */
+    display_info->enable_compositor = FALSE;
+#endif /* HAVE_COMPOSITOR */
     
     initModifiers (display_info->dpy);
 
@@ -397,7 +429,7 @@ initialize (int argc, char **argv, gint compositor_mode)
     nscreens = gdk_display_get_n_screens(display_info->gdisplay);
     for(i = 0; i < nscreens; i++) 
     {
-        ScreenInfo *screen_info = NULL;
+        ScreenInfo *screen_info;
         GdkScreen *gscr;
         
         gscr = gdk_display_get_screen(display_info->gdisplay, i);
@@ -415,25 +447,41 @@ initialize (int argc, char **argv, gint compositor_mode)
         
         if (compositor_mode)
         {
-            compositorManageScreen (screen_info, (compositor_mode == 2));
+            gboolean xfwm4_compositor;
+            
+            xfwm4_compositor = (compositor_mode > 1);
+            if (screen_info->params->use_compositing)
+            {
+                xfwm4_compositor = compositorManageScreen (screen_info);
+            }
+            if (xfwm4_compositor)
+            {
+                /* 
+                   Acquire selection on XFWM4_COMPOSITING_MANAGER to advertise our own
+                   compositing manager (used by WM tweaks to determine whether or not
+                   show the "compositor" tab.
+                 */
+                setAtomIdManagerOwner (display_info, XFWM4_COMPOSITING_MANAGER, 
+                                     screen_info->xroot, screen_info->xfwm4_win);
+            }
         }
 
         sn_init_display (screen_info);
         myDisplayAddScreen (display_info, screen_info);
-        setGnomeProtocols (display_info, screen_info->xroot, screen_info->gnome_win);
-        setHint (display_info, screen_info->xroot, WIN_SUPPORTING_WM_CHECK, screen_info->gnome_win);
-        setHint (display_info, screen_info->xroot, WIN_DESKTOP_BUTTON_PROXY, screen_info->gnome_win);
-        setHint (display_info, screen_info->gnome_win, WIN_DESKTOP_BUTTON_PROXY, screen_info->gnome_win);
+        setGnomeProtocols (display_info, screen_info->xroot, screen_info->xfwm4_win);
+        setHint (display_info, screen_info->xroot, WIN_SUPPORTING_WM_CHECK, screen_info->xfwm4_win);
+        setHint (display_info, screen_info->xroot, WIN_DESKTOP_BUTTON_PROXY, screen_info->xfwm4_win);
+        setHint (display_info, screen_info->xfwm4_win, WIN_DESKTOP_BUTTON_PROXY, screen_info->xfwm4_win);
         getHint (display_info, screen_info->xroot, WIN_WORKSPACE, &ws);
         screen_info->current_ws = (int) ws;
         getGnomeDesktopMargins (display_info, screen_info->xroot, screen_info->gnome_margins);
-        setUTF8StringHint (display_info, screen_info->gnome_win, NET_WM_NAME, "Xfwm4");
-        setNetSupportedHint (display_info, screen_info->xroot, screen_info->gnome_win);
+        setUTF8StringHint (display_info, screen_info->xfwm4_win, NET_WM_NAME, "Xfwm4");
+        setNetSupportedHint (display_info, screen_info->xroot, screen_info->xfwm4_win);
         initNetDesktopInfo (display_info, screen_info->xroot, screen_info->current_ws,
-                                   gdk_screen_get_width (screen_info->gscr), 
-                                   gdk_screen_get_height (screen_info->gscr));
+                                   screen_info->width, 
+                                   screen_info->height);
         workspaceUpdateArea (screen_info);
-        XSetInputFocus (display_info->dpy, screen_info->gnome_win, RevertToPointerRoot, CurrentTime);
+        XSetInputFocus (display_info->dpy, screen_info->xfwm4_win, RevertToPointerRoot, CurrentTime);
 
         clientFrameAll (screen_info);
         
@@ -447,49 +495,45 @@ initialize (int argc, char **argv, gint compositor_mode)
     {
         return -1;
     }
-    display_info->xfilter = xfce_init_event_filter ((gpointer) display_info);
-    xfce_push_event_filter (display_info->xfilter, xfwm4_event_filter, (gpointer) display_info);
+    display_info->xfilter = eventFilterInit ((gpointer) display_info);
+    eventFilterPush (display_info->xfilter, xfwm4_event_filter, (gpointer) display_info);
 
-    client_session = client_session_new (argc, argv, (gpointer) display_info, 
-                                         SESSION_RESTART_IF_RUNNING, 20);
-    client_session->data = (gpointer) client_session;
-    client_session->save_phase_2 = save_phase_2;
-    client_session->die = session_die;
-
-    if (session_init (client_session))
-    {
-        load_saved_session (client_session);
-        return 1;
-    }
-
-    return 0;
+    return sessionStart (argc, argv, display_info);
 }
 
 int
 main (int argc, char **argv)
 {
-    int i;
-    gboolean daemon_mode = FALSE;
-    gint compositor = 2;
+    gboolean daemon_mode;
+    gint compositor;
     int status;
+    int i;
 
+    DBG ("xfwm4 starting");
+
+    daemon_mode = FALSE;
+    compositor = -1;
     for (i = 1; i < argc; i++)
     {
         if (!strcmp (argv[i], "--daemon"))
         {
             daemon_mode = TRUE;
         }
+#ifdef HAVE_COMPOSITOR
         else if (!strncmp (argv[i], "--compositor=", strlen ("--compositor=")))
         {
             compositor = parse_compositor (argv[i]);
         }
+#endif /* HAVE_COMPOSITOR */
         else if (!strcmp(argv[i], "--version") || !strcmp(argv[i], "-V"))
         {
             print_version ();
+	    exit (0);
         }
         else if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-H"))
         {
             print_usage ();
+	    exit (0);
         }
     }
 
