@@ -1,4 +1,4 @@
-/*      $Id: mywindow.c 21694 2006-05-15 21:57:22Z olivier $
+/*      $Id: mywindow.c 22775 2006-08-15 13:54:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -214,7 +214,8 @@ xfwmWindowTemp (ScreenInfo *screen_info, Visual *visual,
                 gint depth, Window parent,
                 xfwmWindow * win, 
                 int x, int y, int width, int height, 
-                long eventmask)
+                long eventmask,
+                gboolean bottom)
 {
     XSetWindowAttributes attributes;
 
@@ -225,7 +226,15 @@ xfwmWindowTemp (ScreenInfo *screen_info, Visual *visual,
                                  InputOnly, CopyFromParent,
                                  CWEventMask | CWOverrideRedirect, 
                                  &attributes);
-    XMapRaised (myScreenGetXDisplay (screen_info), win->window);
+    XMapWindow (myScreenGetXDisplay (screen_info), win->window);
+    if (bottom)
+    {
+        XLowerWindow (myScreenGetXDisplay (screen_info), win->window);
+    }
+    else
+    {
+        XRaiseWindow (myScreenGetXDisplay (screen_info), win->window);
+    }
     XFlush (myScreenGetXDisplay (screen_info));
 
     win->map = TRUE;
