@@ -1,4 +1,4 @@
-/*      $Id: netwm.c 18757 2005-11-07 18:32:33Z olivier $
+/*      $Id: netwm.c 22871 2006-08-24 17:08:39Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
  
@@ -30,24 +30,24 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
 
-#include "display.h"
-#include "screen.h"
-#include "netwm.h"
-#include "misc.h"
 #include "client.h"
+#include "compositor.h"
+#include "display.h"
 #include "frame.h"
 #include "hints.h"
-#include "workspaces.h"
-#include "transients.h"
+#include "misc.h"
+#include "netwm.h"
+#include "screen.h"
 #include "stacking.h"
+#include "transients.h"
+#include "workspaces.h"
 
 void
 clientSetNetState (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     Atom data[16];
     int i;
 
@@ -138,10 +138,10 @@ clientSetNetState (Client * c)
 void
 clientGetNetState (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    int n_atoms = 0;
-    Atom *atoms = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    int n_atoms;
+    Atom *atoms;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGetNetState");
@@ -149,6 +149,8 @@ clientGetNetState (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    atoms = NULL;
+    n_atoms = 0;
 
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_SESSION_MANAGED))
     {
@@ -276,9 +278,9 @@ clientGetNetState (Client * c)
 void
 clientUpdateNetState (Client * c, XClientMessageEvent * ev)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    unsigned long action;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    unsigned long action, mode;
     Atom first;
     Atom second;
 
@@ -292,6 +294,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
     action = ((XEvent *) ev)->xclient.data.l[0];
     first  = ((XEvent *) ev)->xclient.data.l[1];
     second = ((XEvent *) ev)->xclient.data.l[2];
+    mode = 0;
 
     if ((first  == display_info->atoms[NET_WM_STATE_SHADED]) || 
         (second == display_info->atoms[NET_WM_STATE_SHADED]))
@@ -327,7 +330,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             {
                 clientToggleSticky (c, TRUE);
             }
-            frameDraw (c, FALSE, FALSE);
+            frameDraw (c, FALSE);
         }
     }
 
@@ -357,7 +360,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
-                unsigned long mode = 0;
+                mode = 0;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
@@ -368,11 +371,11 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 {
                     mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
                 }
-                clientToggleMaximized (c, mode);
+                clientToggleMaximized (c, mode, TRUE);
             }
             else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
-                unsigned long mode = 0;
+                mode = 0;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
@@ -383,11 +386,11 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 {
                     mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
                 }
-                clientToggleMaximized (c, mode);
+                clientToggleMaximized (c, mode, TRUE);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
-                unsigned long mode = 0;
+                mode = 0;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
@@ -398,7 +401,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 {
                     mode |= WIN_STATE_MAXIMIZED_VERT;
                 }
-                clientToggleMaximized (c, mode);
+                clientToggleMaximized (c, mode, TRUE);
             }
         }
     }
@@ -424,7 +427,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             clientSetNetState (c);
             clientWindowType (c);
         }
-        frameDraw (c, TRUE, FALSE);
+        frameDraw (c, TRUE);
     }
 
     if ((first  == display_info->atoms[NET_WM_STATE_FULLSCREEN]) ||
@@ -480,16 +483,19 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
+                compositorDamageWindow (display_info, c->frame);
                 FLAG_SET (c->flags, CLIENT_FLAG_BELOW);
                 clientUpdateBelowState (c);
             }
             else if ((action == NET_WM_STATE_REMOVE) && FLAG_TEST (c->flags, CLIENT_FLAG_BELOW))
             {
+                compositorDamageWindow (display_info, c->frame);
                 FLAG_UNSET (c->flags, CLIENT_FLAG_BELOW);
                 clientUpdateBelowState (c);
             }
             else if (action == NET_WM_STATE_TOGGLE)
             {
+                compositorDamageWindow (display_info, c->frame);
                 FLAG_TOGGLE (c->flags, CLIENT_FLAG_BELOW);
                 clientUpdateBelowState (c);
             }
@@ -534,7 +540,7 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
             FLAG_TOGGLE (c->flags, CLIENT_FLAG_SKIP_TASKBAR);
             clientSetNetState (c);
         }
-        frameDraw (c, TRUE, FALSE);
+        frameDraw (c, TRUE);
     }
 }
 
@@ -606,10 +612,10 @@ clientUpdateFullscreenState (Client * c)
 void
 clientGetNetWmType (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    int n_atoms = 0;
-    Atom *atoms = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    int n_atoms;
+    Atom *atoms;
     int i;
 
     g_return_if_fail (c != NULL);
@@ -618,7 +624,6 @@ clientGetNetWmType (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
-
     n_atoms = 0;
     atoms = NULL;
 
@@ -670,10 +675,10 @@ clientGetNetWmType (Client * c)
 void
 clientGetInitialNetWmDesktop (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    Client *c2 = NULL;
-    long val = 0;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    Client *c2;
+    long val;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGetInitialNetWmDesktop");
@@ -681,6 +686,7 @@ clientGetInitialNetWmDesktop (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    val = 0;
 
     /* This is to make sure that transient are shown with their "ancestor" window */
     c2 = clientGetTransient (c);
@@ -762,8 +768,11 @@ clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
     if (size < 1)
     {
         XDeleteProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, a);
+        return;
     }
-    else if ((listw = (Window *) malloc ((size + 1) * sizeof (Window))))
+
+    listw = g_new (Window, size + 1);
+    if (listw)
     {
         TRACE ("%i windows in list for %i clients", size, screen_info->client_count);
         for (i = 0, index_dest = listw, index_src = list; i < size;
@@ -774,16 +783,16 @@ clientSetNetClientList (ScreenInfo * screen_info, Atom a, GList * list)
         }
         XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, a, XA_WINDOW, 32, PropModeReplace,
             (unsigned char *) listw, size);
-        free (listw);
+        g_free (listw);
     }
 }
 
 void
 clientGetNetStruts (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
-    gulong *struts = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    gulong *struts;
     int nitems;
     int i;
 
@@ -792,6 +801,7 @@ clientGetNetStruts (Client * c)
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    struts = NULL;
 
     for (i = 0; i < 12; i++)
     {
@@ -844,10 +854,10 @@ clientGetNetStruts (Client * c)
         /* Fill(in values as for partial struts */
         c->struts[TOP_START_X] = c->struts[BOTTOM_START_X] = 0;
         c->struts[TOP_END_X] = c->struts[BOTTOM_END_X] = 
-            gdk_screen_get_width (c->screen_info->gscr);
+            c->screen_info->width;
         c->struts[LEFT_START_Y] = c->struts[RIGHT_START_Y] = 0;
         c->struts[LEFT_END_Y] = c->struts[RIGHT_END_Y] = 
-            gdk_screen_get_height (c->screen_info->gscr);
+            c->screen_info->height;
         
         XFree (struts);
         workspaceUpdateArea (c->screen_info);
@@ -857,16 +867,17 @@ clientGetNetStruts (Client * c)
 void
 clientSetNetActions (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     Atom atoms[6];
-    int i = 0;
+    int i;
 
     g_return_if_fail (c != NULL);
     TRACE ("entering clientSetNetActions");
 
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
+    i = 0;
 
     atoms[i++] = display_info->atoms[NET_WM_ACTION_CLOSE];
     if (CLIENT_CAN_MAXIMIZE_WINDOW (c))
@@ -890,8 +901,8 @@ clientSetNetActions (Client * c)
 void
 clientWindowType (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     netWindowType old_type;
 
     g_return_if_fail (c != NULL);
@@ -925,6 +936,8 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_dock detected");
             c->type = WINDOW_DOCK;
             c->initial_layer = WIN_LAYER_DOCK;
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER |  XFWM_FLAG_HAS_MOVE |
                 XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE | 
@@ -935,6 +948,8 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_toolbar detected");
             c->type = WINDOW_TOOLBAR;
             c->initial_layer = WIN_LAYER_NORMAL;
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_HIDE | XFWM_FLAG_HAS_MAXIMIZE |
                 XFWM_FLAG_HAS_STICK);
@@ -969,6 +984,8 @@ clientWindowType (Client * c)
         else if (c->type_atom == display_info->atoms[NET_WM_WINDOW_TYPE_UTILITY])
         {
             TRACE ("atom net_wm_window_type_utility detected");
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             c->type = WINDOW_UTILITY;
             c->initial_layer = WIN_LAYER_NORMAL;
         }
@@ -977,6 +994,8 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_splash detected");
             c->type = WINDOW_SPLASHSCREEN;
             c->initial_layer = WIN_LAYER_ABOVE_DOCK;
+            FLAG_SET (c->flags,
+                CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_SKIP_TASKBAR);
             FLAG_UNSET (c->xfwm_flags,
                 XFWM_FLAG_HAS_BORDER | XFWM_FLAG_HAS_HIDE |
                 XFWM_FLAG_HAS_MENU | XFWM_FLAG_HAS_MOVE |
