@@ -1,4 +1,4 @@
-/*      $Id: screen.c 20527 2006-03-24 12:26:41Z olivier $
+/*      $Id: screen.c 22775 2006-08-15 13:54:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -26,10 +26,11 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/shape.h>
-#include <gtk/gtk.h>
 #include <glib.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
 
 #ifdef HAVE_RENDER
 #include <X11/extensions/Xrender.h>
@@ -52,10 +53,10 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 #ifdef ENABLE_KDE_SYSTRAY_PROXY
     gchar selection[32];
 #endif
-    long desktop_visible = 0;
-    ScreenInfo *screen_info = NULL;
+    ScreenInfo *screen_info;
     GdkWindow *event_win;
-    PangoLayout *layout = NULL;
+    PangoLayout *layout;
+    long desktop_visible;
     int i;
     
     g_return_val_if_fail (display_info, NULL);
@@ -67,6 +68,8 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     
     screen_info->display_info = display_info;
     screen_info->gscr = gscr;
+    desktop_visible = 0;
+    layout = NULL;
 
     /* Create a GTK window so that we are just like any other GTK application */
     screen_info->gtk_win = gtk_window_new (GTK_WINDOW_POPUP);
@@ -86,7 +89,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     pango_layout_get_pixel_extents (layout, NULL, NULL);
     g_object_unref (G_OBJECT (layout));
 
-    event_win = xfce_add_event_win (gscr, event_mask);
+    event_win = eventFilterAddWin (gscr, event_mask);
     if (!event_win)
     {
         gtk_widget_destroy (screen_info->gtk_win);
@@ -100,6 +103,8 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
     screen_info->screen = gdk_screen_get_number (gscr);
     screen_info->cmap = GDK_COLORMAP_XCOLORMAP(gdk_screen_get_rgb_colormap (gscr));
     screen_info->depth = DefaultDepth (display_info->dpy, screen_info->screen);
+    screen_info->width = WidthOfScreen (screen_info->xscreen);
+    screen_info->height = HeightOfScreen (screen_info->xscreen);
     screen_info->visual = DefaultVisual (display_info->dpy, screen_info->screen);
     screen_info->current_ws = 0;
     screen_info->previous_ws = 0;
@@ -140,17 +145,19 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
                     screen_info->xroot,
                     &screen_info->sidewalk[0], 
                     0, 0, 
-                    1, gdk_screen_get_height (gscr), 
-                    EnterWindowMask);
+                    1, screen_info->height, 
+                    EnterWindowMask,
+                    TRUE);
 
     /*right*/
     xfwmWindowTemp (screen_info,
                     NULL, 0,
                     screen_info->xroot,
                     &screen_info->sidewalk[1], 
-                    gdk_screen_get_width (gscr) - 1, 0, 
-                    1, gdk_screen_get_height (gscr), 
-                    EnterWindowMask);
+                    screen_info->width - 1, 0, 
+                    1, screen_info->height, 
+                    EnterWindowMask,
+                    TRUE);
 
     /*top*/
     xfwmWindowTemp (screen_info,
@@ -158,17 +165,19 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
                     screen_info->xroot,
                     &screen_info->sidewalk[2], 
                     0, 0, 
-                    gdk_screen_get_width (gscr), 1,
-                    EnterWindowMask);
+                    screen_info->width, 1,
+                    EnterWindowMask,
+                    TRUE);
 
     /*bottom*/
     xfwmWindowTemp (screen_info,
                     NULL, 0,
                     screen_info->xroot,
                     &screen_info->sidewalk[3], 
-                    0, gdk_screen_get_height (gscr) - 1, 
-                    gdk_screen_get_width (gscr), 1,
-                    EnterWindowMask);
+                    0, screen_info->height - 1, 
+                    screen_info->width, 1,
+                    EnterWindowMask,
+                    TRUE);
 
     screen_info->xfwm4_win = GDK_WINDOW_XWINDOW (screen_info->gtk_win->window);
 
@@ -221,7 +230,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 ScreenInfo *
 myScreenClose (ScreenInfo *screen_info)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     
     g_return_val_if_fail (screen_info, NULL);
     TRACE ("entering myScreenClose");
@@ -263,7 +272,7 @@ myScreenClose (ScreenInfo *screen_info)
 Display *
 myScreenGetXDisplay (ScreenInfo *screen_info)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     
     g_return_val_if_fail (screen_info, NULL);
     g_return_val_if_fail (screen_info->display_info, NULL);
@@ -294,11 +303,13 @@ myScreenGetGdkWindow (ScreenInfo *screen_info)
 gboolean
 myScreenGrabKeyboard (ScreenInfo *screen_info, Time time)
 {
-    gboolean grab = TRUE;
+    gboolean grab;
 
     g_return_val_if_fail (screen_info, FALSE);
+    
     TRACE ("entering myScreenGrabKeyboard");
 
+    grab = TRUE;
     if (screen_info->key_grabs == 0)
     {
         grab = (XGrabKeyboard (myScreenGetXDisplay (screen_info), 
@@ -316,11 +327,12 @@ myScreenGrabKeyboard (ScreenInfo *screen_info, Time time)
 gboolean
 myScreenGrabPointer (ScreenInfo *screen_info, unsigned int event_mask, Cursor cursor, Time time)
 {
-    gboolean grab = TRUE;
+    gboolean grab;
 
     g_return_val_if_fail (screen_info, FALSE);
     TRACE ("entering myScreenGrabPointer");
 
+    grab = TRUE;
     if (screen_info->pointer_grabs == 0)
     {
         grab = (XGrabPointer (myScreenGetXDisplay (screen_info), 
