@@ -1,4 +1,4 @@
-/*      $Id: screen.h 20527 2006-03-24 12:26:41Z olivier $
+/*      $Id: screen.h 22198 2006-06-25 09:06:45Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <X11/cursorfont.h>
 #include <gtk/gtk.h>
 #include <glib.h>
-#include <libxfcegui4/libxfcegui4.h>
 #include <libxfce4mcs/mcs-client.h>
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
@@ -86,6 +85,8 @@ struct _ScreenInfo
     GdkScreen *gscr;
     Screen *xscreen;
     int depth;
+    int width;
+    int height;
     Visual *visual;
 
     GtkWidget *gtk_win;
@@ -132,22 +133,21 @@ struct _ScreenInfo
 #endif
 
 #ifdef HAVE_COMPOSITOR
-    gboolean compositor_active;
-    gboolean manual_redirect;
-    
     GList *cwindows;
 
     gaussian_conv *gaussianMap;
-    gint gsize;
+    gint gaussianSize;
     guchar *shadowCorner;
     guchar *shadowTop;
-
     
     Picture rootPicture;
     Picture rootBuffer;
     Picture blackPicture;
     Picture rootTile;
     XserverRegion allDamage;
+    
+    guint overlays;
+    gboolean compositor_active;
     gboolean clipChanged;
 #endif /* HAVE_COMPOSITOR */
 };
