@@ -1,16 +1,20 @@
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+/*      $Id: session.c 21727 2006-05-20 11:33:06Z olivier $
+ 
+        This program is free software; you can redistribute it and/or modify
+        it under the terms of the GNU General Public License as published by
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
+ 
+        This program is distributed in the hope that it will be useful,
+        but WITHOUT ANY WARRANTY; without even the implied warranty of
+        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+        GNU General Public License for more details.
+ 
+        You should have received a copy of the GNU General Public License
+        along with this program; if not, write to the Free Software
+        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ 
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  */
 
 /* Initially inspired by xfwm, fvwm2, enlightment and twm implementations */
@@ -19,6 +23,10 @@
 #include <config.h>
 #endif
 
+#include <X11/Xatom.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <glib.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -27,15 +35,16 @@
 #include <stdarg.h>
 #include <sys/types.h>
 #include <signal.h>
-#include <X11/Xatom.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <glib.h>
 
-#include "session.h"
-#include "main.h"
+#include <gtk/gtk.h>
+#include <glib.h>
+#include <libxfcegui4/libxfcegui4.h>
+
+#include "display.h"
+#include "screen.h"
 #include "hints.h"
 #include "client.h"
+#include "session.h"
 
 typedef struct _match
 {
@@ -57,6 +66,7 @@ typedef struct _match
     int old_width;
     int old_height;
     int desktop;
+    int screen;
     unsigned long flags;
     gboolean used;
 }
@@ -98,11 +108,12 @@ escape_quote (gchar * s)
 {
     gchar *ns;
     gchar *idx1, *idx2;
-    gint nbquotes = 0;
-    gint lg = 0;
+    gint nbquotes, lg;
 
     g_return_val_if_fail (s != NULL, NULL);
 
+    nbquotes = 0;
+    lg = 0;
     lg = strlen (s);
     /* First, count quotes in string */
     idx1 = s;
@@ -191,12 +202,13 @@ static gchar *
 getsubstring (gchar * s, gint * length)
 {
     gchar pbrk;
-    gchar *ns;
-    gchar *end, *idx1, *idx2, *skip;
+    gchar *ns, *end, *idx1, *idx2, *skip;
     gint lg;
-    gboolean finished = FALSE, backslash = FALSE;
+    gboolean finished, backslash;
 
     lg = *length = 0;
+    finished = FALSE;
+    backslash = FALSE;
     g_return_val_if_fail (s != NULL, NULL);
 
     end = skip = s;
@@ -249,98 +261,118 @@ getsubstring (gchar * s, gint * length)
     return ns;
 }
 
+static void
+sessionSaveScreen (ScreenInfo *screen_info, FILE *f)
+{
+    DisplayInfo *display_info;
+    Client *c;
+    char *client_id, *window_role;
+    char **wm_command;
+    int wm_command_count;
+    gint client_idx;
+
+    display_info = screen_info->display_info;
+    wm_command_count = 0;
+    wm_command = NULL;
+    window_role = NULL;
+    client_id = NULL;
+
+    for (c = screen_info->clients, client_idx = 0; client_idx < screen_info->client_count;
+        c = c->next, client_idx++)
+    {
+        if (c->client_leader != None)
+        {
+            getWindowRole (display_info, c->window, &window_role);
+        }
+        else
+        {
+            window_role = NULL;
+        }
+
+        fprintf (f, "[CLIENT] 0x%lx\n", c->window);
+
+        getClientID (display_info, c->window, &client_id);
+        if (client_id)
+        {
+            fprintf (f, "  [CLIENT_ID] %s\n", client_id);
+            XFree (client_id);
+            client_id = NULL;
+        }
+
+        if (c->client_leader)
+        {
+            fprintf (f, "  [CLIENT_LEADER] 0x%lx\n", c->client_leader);
+        }
+
+        if (window_role)
+        {
+            fprintf (f, "  [WINDOW_ROLE] %s\n", window_role);
+            XFree (window_role);
+            window_role = NULL;
+        }
+
+        if (c->class.res_class)
+        {
+            fprintf (f, "  [RES_NAME] %s\n", c->class.res_name);
+        }
+
+        if (c->class.res_name)
+        {
+            fprintf (f, "  [RES_CLASS] %s\n", c->class.res_class);
+        }
+
+        if (c->name)
+        {
+            fprintf (f, "  [WM_NAME] %s\n", c->name);
+        }
+
+        wm_command_count = 0;
+        getWindowCommand (display_info, c->window, &wm_command, &wm_command_count);
+        if ((wm_command_count > 0) && (wm_command))
+        {
+            gint j;
+            fprintf (f, "  [WM_COMMAND] (%i)", wm_command_count);
+            for (j = 0; j < wm_command_count; j++)
+            {
+                gchar *escaped_string;
+                escaped_string = escape_quote (wm_command[j]);
+                fprintf (f, " \"%s\"", escaped_string);
+                g_free (escaped_string);
+            }
+            fprintf (f, "\n");
+            XFreeStringList (wm_command);
+            wm_command = NULL;
+            wm_command_count = 0;
+        }
+
+        fprintf (f, "  [GEOMETRY] (%i,%i,%i,%i)\n", c->x, c->y, c->width,
+            c->height);
+        fprintf (f, "  [GEOMETRY-MAXIMIZED] (%i,%i,%i,%i)\n", c->old_x,
+            c->old_y, c->old_width, c->old_height);
+        fprintf (f, "  [SCREEN] %i\n", screen_info->screen);
+        fprintf (f, "  [DESK] %i\n", c->win_workspace);
+        fprintf (f, "  [FLAGS] 0x%lx\n", FLAG_TEST (c->flags,
+                CLIENT_FLAG_STICKY | CLIENT_FLAG_ICONIFIED |
+                CLIENT_FLAG_SHADED | CLIENT_FLAG_MAXIMIZED |
+                CLIENT_FLAG_NAME_CHANGED));
+    }
+}
+
 gboolean
-sessionSaveWindowStates (gchar * filename)
+sessionSaveWindowStates (DisplayInfo *display_info, gchar * filename)
 {
     FILE *f;
-    Client *c;
-    gint client_idx;
-    char *client_id = NULL;
-    char *window_role = NULL;
-    int wm_command_count = 0;
-    char **wm_command = NULL;
+    GSList *screens;
 
     g_return_val_if_fail (filename != NULL, FALSE);
+    g_return_val_if_fail (display_info != NULL, FALSE);
 
     if ((f = fopen (filename, "w")))
     {
-        for (c = clients, client_idx = 0; client_idx < client_count;
-            c = c->next, client_idx++)
+        for (screens = display_info->screens; screens; screens = g_slist_next (screens))
         {
-            if (c->client_leader != None)
-            {
-                getWindowRole (dpy, c->client_leader, &window_role);
-            }
-            else
-            {
-                window_role = NULL;
-            }
-
-            fprintf (f, "[CLIENT] 0x%lx\n", c->window);
-
-            getClientID (dpy, c->window, &client_id);
-            if (client_id)
-            {
-                fprintf (f, "  [CLIENT_ID] %s\n", client_id);
-                XFree (client_id);
-                client_id = NULL;
-            }
-
-            if (c->client_leader)
-            {
-                fprintf (f, "  [CLIENT_LEADER] 0x%lx\n", c->client_leader);
-            }
-
-            if (window_role)
-            {
-                fprintf (f, "  [WINDOW_ROLE] %s\n", window_role);
-                XFree (window_role);
-                window_role = NULL;
-            }
-
-            if (c->class.res_class)
-            {
-                fprintf (f, "  [RES_NAME] %s\n", c->class.res_name);
-            }
-
-            if (c->class.res_name)
-            {
-                fprintf (f, "  [RES_CLASS] %s\n", c->class.res_class);
-            }
-
-            if (c->name)
-            {
-                fprintf (f, "  [WM_NAME] %s\n", c->name);
-            }
-
-            wm_command_count = 0;
-            getWindowCommand (dpy, c->window, &wm_command, &wm_command_count);
-            if ((wm_command_count > 0) && (wm_command))
-            {
-                gint j;
-                fprintf (f, "  [WM_COMMAND] (%i)", wm_command_count);
-                for (j = 0; j < wm_command_count; j++)
-                {
-                    gchar *escaped_string;
-                    escaped_string = escape_quote (wm_command[j]);
-                    fprintf (f, " \"%s\"", escaped_string);
-                    g_free (escaped_string);
-                }
-                fprintf (f, "\n");
-                XFreeStringList (wm_command);
-                wm_command = NULL;
-                wm_command_count = 0;
-            }
-
-            fprintf (f, "  [GEOMETRY] (%i,%i,%i,%i)\n", c->x, c->y, c->width,
-                c->height);
-            fprintf (f, "  [GEOMETRY-MAXIMIZED] (%i,%i,%i,%i)\n", c->old_x,
-                c->old_y, c->old_width, c->old_height);
-            fprintf (f, "  [DESK] %i\n", c->win_workspace);
-            fprintf (f, "  [FLAGS] 0x%lx\n", CLIENT_FLAG_TEST (c,
-                    CLIENT_FLAG_STICKY | CLIENT_FLAG_HIDDEN |
-                    CLIENT_FLAG_SHADED | CLIENT_FLAG_MAXIMIZED |
-                    CLIENT_FLAG_NAME_CHANGED));
+            ScreenInfo *screen_info_n = (ScreenInfo *) screens->data;
+            sessionSaveScreen (screen_info_n, f);
         }
         fclose (f);
         return TRUE;
@@ -381,11 +413,10 @@ sessionLoadWindowStates (gchar * filename)
                 matches[num_match - 1].height = 100;
                 matches[num_match - 1].old_x = matches[num_match - 1].x;
                 matches[num_match - 1].old_y = matches[num_match - 1].y;
-                matches[num_match - 1].old_width =
-                    matches[num_match - 1].width;
-                matches[num_match - 1].old_height =
-                    matches[num_match - 1].height;
+                matches[num_match - 1].old_width = matches[num_match - 1].width;
+                matches[num_match - 1].old_height = matches[num_match - 1].height;
                 matches[num_match - 1].desktop = 0;
+                matches[num_match - 1].screen = 0;
                 matches[num_match - 1].used = FALSE;
                 matches[num_match - 1].flags = 0;
             }
@@ -402,6 +433,10 @@ sessionLoadWindowStates (gchar * filename)
                     &matches[num_match - 1].old_width,
                     &matches[num_match - 1].old_height);
             }
+            else if (!strcmp (s1, "[SCREEN]"))
+            {
+                sscanf (s, "%*s %i", &matches[num_match - 1].screen);
+            }
             else if (!strcmp (s1, "[DESK]"))
             {
                 sscanf (s, "%*s %i", &matches[num_match - 1].desktop);
@@ -442,22 +477,17 @@ sessionLoadWindowStates (gchar * filename)
             }
             else if (!strcmp (s1, "[WM_COMMAND]"))
             {
-                sscanf (s, "%*s (%i)%n",
-                    &matches[num_match - 1].wm_command_count, &pos);
-                matches[num_match - 1].wm_command =
-                    g_new (gchar *,
-                    matches[num_match - 1].wm_command_count + 1);
+                sscanf (s, "%*s (%i)%n", &matches[num_match - 1].wm_command_count, &pos);
+                matches[num_match - 1].wm_command = g_new (gchar *, matches[num_match - 1].wm_command_count + 1);
                 for (i = 0; i < matches[num_match - 1].wm_command_count; i++)
                 {
                     gchar *substring;
                     substring = getsubstring (s + pos, &pos1);
                     pos += pos1;
-                    matches[num_match - 1].wm_command[i] =
-                        unescape_quote (substring);
+                    matches[num_match - 1].wm_command[i] = unescape_quote (substring);
                     g_free (substring);
                 }
-                matches[num_match - 1].wm_command[matches[num_match -
-                        1].wm_command_count] = NULL;
+                matches[num_match - 1].wm_command[matches[num_match - 1].wm_command_count] = NULL;
             }
         }
         fclose (f);
@@ -470,6 +500,7 @@ void
 sessionFreeWindowStates (void)
 {
     int i;
+
     for (i = 0; i < num_match; i++)
     {
         if (matches[i].client_id)
@@ -521,23 +552,31 @@ sessionFreeWindowStates (void)
 static gboolean
 matchWin (Client * c, Match * m)
 {
-    char *client_id = NULL;
-    char *window_role = NULL;
-    int wm_command_count = 0;
-    char **wm_command = NULL;
-    gboolean found;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    char *client_id;
+    char *window_role;
+    char **wm_command;
+    int wm_command_count;
     int i;
+    gboolean found;
 
     g_return_val_if_fail (c != NULL, FALSE);
 
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+    wm_command = NULL;
+    window_role = NULL;
+    client_id = NULL;
     found = FALSE;
-    getClientID (dpy, c->window, &client_id);
+
+    getClientID (display_info, c->window, &client_id);
     if (xstreq (client_id, m->client_id))
     {
         /* client_id's match */
         if (c->client_leader != None)
         {
-            getWindowRole (dpy, c->client_leader, &window_role);
+            getWindowRole (display_info, c->window, &window_role);
         }
         else
         {
@@ -550,11 +589,12 @@ matchWin (Client * c, Match * m)
         }
         else
         {
-            /* Compare res_class, res_name and WM_NAME, unless the
+            /* 
+             * Compare res_class, res_name and WM_NAME, unless the
              * WM_NAME has changed
              */
             if (xstreq (c->class.res_name, m->res_name)
-                && (CLIENT_FLAG_TEST (c, CLIENT_FLAG_NAME_CHANGED)
+                && (FLAG_TEST (c->flags, CLIENT_FLAG_NAME_CHANGED)
                     || (m->flags & CLIENT_FLAG_NAME_CHANGED)
                     || xstreq (c->name, m->wm_name)))
             {
@@ -563,14 +603,12 @@ matchWin (Client * c, Match * m)
                     /* If we have a client_id, we don't compare
                        WM_COMMAND, since it will be different. */
                     found = TRUE;
-
                 }
                 else
                 {
                     /* for non-SM-aware clients we also compare WM_COMMAND */
                     wm_command_count = 0;
-                    getWindowCommand (dpy, c->window, &wm_command,
-                        &wm_command_count);
+                    getWindowCommand (display_info, c->window, &wm_command, &wm_command_count);
                     if (wm_command_count == m->wm_command_count)
                     {
                         for (i = 0; i < wm_command_count; i++)
@@ -584,7 +622,8 @@ matchWin (Client * c, Match * m)
                             found = TRUE;
                         }
                     }           /* if (wm_command_count ==... */
-                    /* We have to deal with a now-SM-aware client, it means that it won't probably
+                    /* 
+                     * We have to deal with a now-SM-aware client, it means that it won't probably
                      * restore its state in a proper manner.
                      * Thus, we also mark all other instances of this application as used, to avoid
                      * dummy side effects in case we found a matching entry.
@@ -595,8 +634,7 @@ matchWin (Client * c, Match * m)
                         {
                             if (!(matches[i].used) && !(&matches[i] == m)
                                 && (m->client_leader)
-                                && (matches[i].client_leader ==
-                                    m->client_leader))
+                                && (matches[i].client_leader == m->client_leader))
                             {
                                 matches[i].used = TRUE;
                             }
@@ -637,7 +675,7 @@ sessionMatchWinToSM (Client * c)
     g_return_val_if_fail (c != NULL, FALSE);
     for (i = 0; i < num_match; i++)
     {
-        if (!matches[i].used && matchWin (c, &matches[i]))
+        if (!matches[i].used && (c->screen_info->screen == matches[i].screen) && matchWin (c, &matches[i]))
         {
             matches[i].used = TRUE;
             c->x = matches[i].x;
@@ -649,13 +687,102 @@ sessionMatchWinToSM (Client * c)
             c->old_width = matches[i].old_width;
             c->old_height = matches[i].old_height;
             c->win_workspace = matches[i].desktop;
-            CLIENT_FLAG_SET (c,
+            FLAG_SET (c->flags,
                 matches[i].
                 flags & (CLIENT_FLAG_STICKY | CLIENT_FLAG_SHADED |
-                    CLIENT_FLAG_MAXIMIZED | CLIENT_FLAG_HIDDEN));
-            CLIENT_FLAG_SET (c, CLIENT_FLAG_WORKSPACE_SET);
+                    CLIENT_FLAG_MAXIMIZED | CLIENT_FLAG_ICONIFIED));
+            FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
             return TRUE;
         }
     }
     return FALSE;
 }
+
+static char *
+sessionBuildFilename(SessionClient *client_session)
+{
+    gchar *filename, *path, *file;
+    GError *error;
+
+    path = xfce_resource_save_location (XFCE_RESOURCE_CACHE, "sessions", FALSE);
+
+    error = NULL;
+    if (!xfce_mkdirhier(path, 0700, &error)) 
+    {
+        g_warning("Unable to create session dir %s: %s", path, error->message);
+        g_error_free (error);
+        g_free (path);
+        return NULL;
+    }
+
+    file = g_strdup_printf("xfwm4-%s", client_session->given_client_id);
+    filename = g_build_filename (path, file, NULL);
+    g_free (file);
+    g_free (path);
+    
+    return filename;
+}
+
+static void
+sessionLoad (DisplayInfo *display_info)
+{
+    SessionClient *session;
+    gchar *filename;
+
+    session = display_info->session;
+    filename = sessionBuildFilename(session);
+    if (filename)
+    {
+        sessionLoadWindowStates (filename);
+        g_free (filename);
+    }
+}
+
+static void
+sessionSavePhase2 (gpointer data)
+{
+    DisplayInfo *display_info;
+    SessionClient *session;
+    gchar *filename;
+
+    display_info = (DisplayInfo *) data;
+    session = display_info->session;
+    filename = sessionBuildFilename(session);
+    if (filename)
+    {
+        sessionSaveWindowStates (display_info, filename);
+        g_free (filename);
+    }
+}
+
+static void
+sessionDie (gpointer data)
+{
+    DisplayInfo *display_info;
+
+    display_info = (DisplayInfo *) data;
+    display_info->quit = TRUE;
+    gtk_main_quit ();
+}
+
+int
+sessionStart (int argc, char **argv, DisplayInfo *display_info)
+{
+    SessionClient *session;
+    
+    display_info->session = client_session_new (argc, argv, (gpointer) display_info, 
+                                                SESSION_RESTART_IF_RUNNING, 20);
+    session = display_info->session;
+    session->data = (gpointer) display_info;
+    session->save_phase_2 = sessionSavePhase2;
+    session->die = sessionDie;
+
+    if (session_init (session))
+    {
+        sessionLoad (display_info);
+        return 1;
+    }
+
+    return 0;
+}
+
