@@ -1,4 +1,4 @@
-/*      $Id: settings.c 22306 2006-07-09 09:07:25Z olivier $
+/*      $Id: settings.c 22775 2006-08-15 13:54:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -439,6 +439,10 @@ notify_cb (const char *name, const char *channel_name, McsAction action, McsSett
                     {
                         screen_info->params->toggle_workspaces = setting->data.v_int;
                     }
+                    else if (!strcmp (name, "Xfwm/UnredirectOverlays"))
+                    {
+                        screen_info->params->unredirect_overlays = setting->data.v_int;
+                    }
                     else if (!strcmp (name, "Xfwm/UseCompositing"))
                     {
                         screen_info->params->use_compositing = setting->data.v_int;
@@ -830,6 +834,12 @@ loadMcsData (ScreenInfo *screen_info, Settings *rc)
             setBooleanValueFromInt ("toggle_workspaces", setting->data.v_int, rc);
             mcs_setting_free (setting);
         }
+        if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/UnredirectOverlays", CHANNEL5,
+                &setting) == MCS_SUCCESS)
+        {
+            setBooleanValueFromInt ("unredirect_overlays", setting->data.v_int, rc);
+            mcs_setting_free (setting);
+        }
         if (mcs_client_get_setting (screen_info->mcs_client, "Xfwm/UseCompositing", CHANNEL5,
                 &setting) == MCS_SUCCESS)
         {
@@ -1129,6 +1139,7 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     strncpy (screen_info->params->button_layout, getValue ("button_layout", rc), 7);
     screen_info->params->button_spacing = TOINT (getValue ("button_spacing", rc));
     screen_info->params->button_offset = TOINT (getValue ("button_offset", rc));
+    screen_info->params->maximized_offset = TOINT (getValue ("maximized_offset", rc));
     screen_info->params->title_vertical_offset_active =
         TOINT (getValue ("title_vertical_offset_active", rc));
     screen_info->params->title_vertical_offset_inactive =
@@ -1288,6 +1299,7 @@ loadSettings (ScreenInfo *screen_info)
         {"margin_left", NULL, FALSE},
         {"margin_right", NULL, FALSE},
         {"margin_top", NULL, FALSE},
+        {"maximized_offset", NULL, TRUE},
         {"move_opacity", NULL, TRUE},
         {"placement_ratio", NULL, TRUE},
         {"popup_opacity", NULL, TRUE},
@@ -1319,6 +1331,7 @@ loadSettings (ScreenInfo *screen_info)
         {"title_vertical_offset_active", NULL, TRUE},
         {"title_vertical_offset_inactive", NULL, TRUE},
         {"toggle_workspaces", NULL, TRUE},
+        {"unredirect_overlays", NULL, TRUE},
         {"use_compositing", NULL, TRUE},
         {"workspace_count", NULL, TRUE},
         {"wrap_cycle", NULL, TRUE},
@@ -1491,6 +1504,8 @@ loadSettings (ScreenInfo *screen_info)
 
     screen_info->params->toggle_workspaces =
         !g_ascii_strcasecmp ("true", getValue ("toggle_workspaces", rc));
+    screen_info->params->unredirect_overlays =
+        !g_ascii_strcasecmp ("true", getValue ("unredirect_overlays", rc));
     screen_info->params->use_compositing =
         !g_ascii_strcasecmp ("true", getValue ("use_compositing", rc));
     screen_info->params->wrap_workspaces =
