@@ -1,4 +1,4 @@
-/*      $Id: settings.h 22306 2006-07-09 09:07:25Z olivier $
+/*      $Id: settings.h 22775 2006-08-15 13:54:19Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -154,6 +154,7 @@ struct _XfwmParams
     int easy_click;
     int frame_opacity;
     int inactive_opacity;
+    int maximized_offset;
     int move_opacity;
     int placement_ratio;
     int popup_opacity;
@@ -192,6 +193,7 @@ struct _XfwmParams
     gboolean title_vertical_offset_active;
     gboolean title_vertical_offset_inactive;
     gboolean toggle_workspaces;
+    gboolean unredirect_overlays;
     gboolean use_compositing;
     gboolean wrap_cycle;
     gboolean wrap_layout;
