@@ -1,4 +1,4 @@
-/*      $Id: spinning_cursor.c 19935 2006-02-18 08:21:05Z olivier $
+/*      $Id: spinning_cursor.c 22466 2006-07-16 17:03:26Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 
 #include <X11/Xlib.h>
 
+#include "spinning_cursor.h"
+
 static const unsigned char xlib_spinning_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00,
     0x0c, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00,
@@ -56,10 +58,10 @@ static const unsigned char xlib_spinning_mask_bits[] = {
 Cursor
 cursorCreateSpinning (Display * dpy)
 {
-    Pixmap cursor = None;
-    Pixmap mask = None;
+    Pixmap cursor;
+    Pixmap mask;
     XColor fg, bg;
-    Cursor xcursor = None;
+    Cursor xcursor;
 
     fg.pixel = 0;
     fg.red = 0;
