@@ -1,4 +1,4 @@
-/*      $Id: tabwin.c 20501 2006-03-20 06:39:36Z olivier $
+/*      $Id: tabwin.c 22407 2006-07-11 21:47:03Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
 #include "inline-tabwin-icon.h"
 #include "icons.h"
 #include "focus.h"
@@ -44,7 +43,7 @@
 static GdkColor *
 get_color (GtkWidget * win, GtkStateType state_type)
 {
-    GtkStyle *style = NULL;
+    GtkStyle *style;
 
     g_return_val_if_fail (win != NULL, NULL);
     g_return_val_if_fail (GTK_IS_WIDGET (win), NULL);
@@ -163,28 +162,35 @@ createWindowIcon (Client * c)
 }
 
 static GtkWidget *
-createWindowlist (GdkScreen * scr, Client * c, unsigned int cycle_range, Tabwin * t)
+createWindowlist (GdkScreen * scr, Client * current, Client * new, unsigned int cycle_range, Tabwin * t)
 {
+    ScreenInfo *screen_info;
+    Client *c2;
     GdkRectangle monitor_sz;
     GtkWidget *windowlist, *icon;
     GList *next;
-    Client *c2 = NULL;
-    ScreenInfo *screen_info;
     unsigned int grid_cols;
     unsigned int n_clients;
     unsigned int grid_rows;
-    int i = 0, packpos = 0;
+    int i, packpos;
+    int msx, msy;
     gint monitor;
 
-    g_return_val_if_fail (c != NULL, NULL);
+    g_return_val_if_fail (current != NULL, NULL);
+
+    i = 0;
+    packpos = 0;
+    c2 = NULL;
+    next = NULL;
 
     /* calculate the wrapping */
-    screen_info = c->screen_info;
+    screen_info = current->screen_info;
     n_clients = screen_info->client_count;
     
     g_return_val_if_fail (n_clients > 0, NULL);
     
-    xfce_gdk_display_locate_monitor_with_pointer (screen_info->display_info->gdisplay, &monitor);
+    getMouseXY (screen_info, screen_info->xroot, &msx, &msy);
+    monitor = find_monitor_at_point (scr, msx, msy);
     gdk_screen_get_monitor_geometry (scr, monitor, &monitor_sz);
     
     /* add the width of the border on each side */
@@ -195,7 +201,7 @@ createWindowlist (GdkScreen * scr, Client * c, unsigned int cycle_range, Tabwin
     t->grid_cols = grid_cols;
     t->grid_rows = grid_rows;
     /* pack the client icons */
-    for (c2 = c, i = 0; c2 && i < n_clients; i++, c2 = c2->next)
+    for (c2 = current, i = 0; c2 && i < n_clients; i++, c2 = c2->next)
     {
         if (!clientSelectMask (c2, cycle_range, WINDOW_REGULAR_FOCUSABLE))
             continue;
@@ -206,23 +212,21 @@ createWindowlist (GdkScreen * scr, Client * c, unsigned int cycle_range, Tabwin
             GTK_FILL, GTK_FILL, 7, 7);
         packpos++;
         t->head = g_list_append (t->head, icon);
+        if (c2 == new)
+        {
+            next = g_list_last (t->head);
+        }
     }
-
-    next = g_list_next (t->head);
-    if (!next)
-    {
-        next = t->head;
-    }
-    t->current = next;
     if (next)
     {
         tabwinSetSelected (t, next->data);
     }
+    t->current = next;
     return windowlist;
 }
 
 Tabwin *
-tabwinCreate (GdkScreen * scr, Client * c, unsigned int cycle_range, gboolean display_workspace)
+tabwinCreate (GdkScreen * scr, Client * current, Client * new, unsigned int cycle_range, gboolean display_workspace)
 {
     Tabwin *tabwin;
     GtkWidget *frame;
@@ -274,7 +278,7 @@ tabwinCreate (GdkScreen * scr, Client * c, unsigned int cycle_range, gboolean di
     gtk_box_pack_start (GTK_BOX (vbox), tabwin->label, TRUE, TRUE, 0);
     gtk_widget_set_size_request (GTK_WIDGET (tabwin->label), 240, -1);
 
-    windowlist = createWindowlist (scr, c, cycle_range, tabwin);
+    windowlist = createWindowlist (scr, current, new, cycle_range, tabwin);
     tabwin->container = windowlist;
     gtk_container_add (GTK_CONTAINER (frame), windowlist);
 
@@ -299,16 +303,14 @@ tabwinCreate (GdkScreen * scr, Client * c, unsigned int cycle_range, gboolean di
 Client *
 tabwinGetSelected (Tabwin * t)
 {
-    Client *c = NULL;
-
     g_return_val_if_fail (t != NULL, NULL);
 
     if ((t->current) && (t->current->data))
     {
-        c = (Client *) g_object_get_data (G_OBJECT (t->current->data), "client-ptr-val");
+        return (Client *) g_object_get_data (G_OBJECT (t->current->data), "client-ptr-val");
     }
 
-    return c;
+    return NULL;
 }
 
 Client *
