@@ -1,4 +1,4 @@
-/*      $Id: tabwin.h 11914 2004-08-22 22:18:50Z olivier $
+/*      $Id: tabwin.h 22407 2006-07-11 21:47:03Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -32,14 +32,27 @@
 typedef struct _Tabwin Tabwin;
 struct _Tabwin
 {
+    /* The below must be freed when destroying */
     GtkWidget *window;
-    GtkWidget *image;
+    GList *head;
+
+    /* these don't have to be */
     GtkWidget *class;
     GtkWidget *label;
+    GtkWidget *container;
+    GList *current;
+    gulong selected_callback;
+    gboolean display_workspace;
+
+    int grid_cols;
+    int grid_rows;
 };
 
-Tabwin *tabwinCreate (GdkScreen *, GdkPixbuf *, const gchar *, const gchar *);
-void tabwinSetLabel (Tabwin *, GdkPixbuf *, const gchar *, const gchar *);
-void tabwinDestroy (Tabwin *);
+Tabwin *tabwinCreate (GdkScreen *, Client *, Client *, unsigned int, gboolean);
+Client *tabwinGetSelected (Tabwin *);
+Client *tabwinSelectNext (Tabwin *);
+Client *tabwinSelectPrev (Tabwin *);
+Client *tabwinRemoveClient (Tabwin *, Client *);
+void    tabwinDestroy (Tabwin *);
 
 #endif /* INC_TABWIN_H */
