@@ -1,4 +1,4 @@
-/*      $Id: workspaces.c 20527 2006-03-24 12:26:41Z olivier $
+/*      $Id: workspaces.c 22939 2006-08-29 21:57:14Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -45,9 +45,10 @@
 static void
 workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
 {
-    NetWmDesktopLayout l = screen_info->desktop_layout;
+    NetWmDesktopLayout l;
     int major_length, minor_length, tmp;
 
+    l = screen_info->desktop_layout;
     if (l.orientation == NET_WM_ORIENTATION_HORZ)
     {
         major_length = l.cols;
@@ -94,9 +95,10 @@ workspaceGetPosition (ScreenInfo *screen_info, int n, int * row, int * col)
 static int
 workspaceGetNumber (ScreenInfo *screen_info, int row, int col)
 {
-    NetWmDesktopLayout l = screen_info->desktop_layout;
+    NetWmDesktopLayout l;
     int major_length, minor_length, n, tmp;
 
+    l = screen_info->desktop_layout;
     if (l.orientation == NET_WM_ORIENTATION_HORZ)
     {
         major_length = l.cols;
@@ -172,7 +174,7 @@ modify_with_wrap (int value, int by, int limit, gboolean wrap)
 
 /* returns TRUE if the workspace was changed, FALSE otherwise */
 gboolean
-workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
+workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c)
 {
     int row, col, newrow, newcol, previous_ws, n;
 
@@ -189,7 +191,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
     previous_ws = screen_info->current_ws;
     if ((n >= 0) && (n < screen_info->workspace_count))
     {
-        workspaceSwitch(screen_info, n, c2, TRUE);
+        workspaceSwitch(screen_info, n, c, TRUE);
     }
     else if (screen_info->params->wrap_layout)
     {
@@ -218,7 +220,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
 
             n = workspaceGetNumber(screen_info, newrow, newcol);
         }
-        workspaceSwitch(screen_info, n, c2, TRUE);
+        workspaceSwitch(screen_info, n, c, TRUE);
     }
 
     return (screen_info->current_ws != previous_ws);
@@ -227,8 +229,8 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
 void
 workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean update_focus)
 {
-    DisplayInfo *display_info = screen_info->display_info;
-    Client *c, *new_focus = NULL;
+    DisplayInfo *display_info;
+    Client *c, *new_focus;
     Client *previous;
     GList *index;
     GList *list_hide;
@@ -239,6 +241,8 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
     
     TRACE ("entering workspaceSwitch");
 
+    display_info = screen_info->display_info;
+    new_focus = NULL;
     if ((new_ws == screen_info->current_ws) && (screen_info->params->toggle_workspaces))
     {
         new_ws = screen_info->previous_ws;
@@ -368,7 +372,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2, gboolean upda
 }
 
 void
-workspaceSetNames (ScreenInfo * screen_info, char **names, int items)
+workspaceSetNames (ScreenInfo * screen_info, gchar **names, int items)
 {
     if (screen_info->workspace_names)
     {
@@ -382,7 +386,7 @@ workspaceSetNames (ScreenInfo * screen_info, char **names, int items)
 void
 workspaceSetCount (ScreenInfo * screen_info, int count)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     Client *c;
     int i;
 
@@ -414,9 +418,7 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
         workspaceSwitch (screen_info, count - 1, NULL, TRUE);
     }
     setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
-                    gdk_screen_get_width (screen_info->gscr),
-                    gdk_screen_get_height (screen_info->gscr),
-                    screen_info->margins);
+                    screen_info->width, screen_info->height, screen_info->margins);
     /* Recompute the layout based on the (changed) number of desktops */
     getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, 
                      &screen_info->desktop_layout);
@@ -425,7 +427,7 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
 void
 workspaceUpdateArea (ScreenInfo *screen_info)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     Client *c;
     int prev_top;
     int prev_left;
@@ -468,9 +470,7 @@ workspaceUpdateArea (ScreenInfo *screen_info)
     {
         TRACE ("Margins have changed, updating net_workarea");
         setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
-                        gdk_screen_get_width (screen_info->gscr),
-                        gdk_screen_get_height (screen_info->gscr),
-                        screen_info->margins);
+                        screen_info->width, screen_info->height, screen_info->margins);
         /* Also prevent windows from being off screen, just like when screen is resized */
         clientScreenResize(screen_info);
     }
